      PROGRAM JACOBIE
C     ----------------------------------------------------------------
C     Alg11'3.for   FORTRAN program for implementing Algorithm 11.3
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 11.3 (Jacobi Iteration for Eigenvalues & Eigenvectors)
C     Section   11.3, Jacobi's Method, Page 571
C     ----------------------------------------------------------------
      PARAMETER(MaxR = 4, MaxS = 50)
      CHARACTER Ans*1,Ach*1,Mess*80
      INTEGER CountR,CountS
      INTEGER I,InRC,Inum,N,P,Q,Row
      REAL Apq,C,Epsilon,MaxA,S,Rnum,T
      DIMENSION Row(1:MaxR)
      REAL XP,XQ,YP,YQ
      DIMENSION XP(1:MaxR),XQ(1:MaxR)
      DIMENSION YP(1:MaxR),YQ(1:MaxR)
      REAL A,A1,V
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      DIMENSION V(1:MaxR,1:MaxR)
      INTEGER Method,Cyclic,Original
      INTEGER Proc,Auto,Manual,Observe
      EXTERNAL RMS
      Original = 1
      Cyclic = 2
      Auto = 1
      Manual = 2
      Observe = 3
      CALL MESSAGE(Meth)
      CALL INPUTS(A,A1,N,Epsilon,InRC)
      CALL PROCESSES(Proc)
      CALL INITIALV(V,N)
      CALL MAXPQ(A,N,P,Q,MaxA)
      IF (Meth.EQ.Original) THEN
        CALL OJacob(A,V,N,Epsilon,CountR,Proc)
      ENDIF
      IF (Meth.EQ.Cyclic) THEN
        CALL CJacob(A,V,N,Epsilon,CountS,Proc)
      ENDIF
      CALL PrintResults(A,A1,V,N,Epsilon,MaxA,CountS,Meth)
      END

      SUBROUTINE Routput(P,Q,C,S,Apq)
      INTEGER P,Q
      REAL Apq,C,S
      WRITE(9,*)' '
      WRITE(9,*)'The rotation matrix for zeroing out'
      WRITE(9,*)'A(',P, ',',Q, ') =',Apq,'  is'
      WRITE(9,*)' '
      WRITE(9,*)'    (          )   (                     )'
      WRITE(9,*)'    (  c    s  )   (', C, '   ', S, '  )'
      WRITE(9,*)'R = (          ) = (                     )'
      WRITE(9,*)'    ( -s    c  )   (', -S, '   ', C , ' )'
      WRITE(9,*)'    (          )   (                     )'
      RETURN
      END

      SUBROUTINE MATPRINT(A,N)
      PARAMETER(MaxR = 4)
      INTEGER Col,N,Row
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      WRITE(9,*)' '
      DO 10 Row=1,N
        WRITE(9,999) ( A(Row,Col), Col=1,N )
10    CONTINUE
999   FORMAT(1X,5F15.7,4X)
      RETURN
      END
      
      SUBROUTINE ZEROPQ(A,V,C,S,N,P,Q)
      PARAMETER(MaxR = 4)
      REAL A,V
      DIMENSION A(1:MaxR,1:MaxR),V(1:MaxR,1:MaxR)
      INTEGER I,J,N,P,Q,Pm,Qm
      REAL C,S,T,Theta
      REAL XP,XQ,YP,YQ
      DIMENSION XP(1:MaxR),XQ(1:MaxR)
      DIMENSION YP(1:MaxR),YQ(1:MaxR)
      Theta = (A(Q,Q) - A(P,P))/(2*A(P,Q))
      IF (Theta.LT.1E19) THEN
        T = 1/(ABS(Theta) + SQRT(Theta*Theta+1))
      ELSE
        T = 1/ABS(2*Theta)
      ENDIF
      IF (Theta.LT.0) THEN
        T = -T
      ENDIF
      C = 1/SQRT(T*T+1)
      S = C*T
      DO I=1,N
        XP(I) = A(I,P)
        XQ(I) = A(I,Q)
      ENDDO
      A(P,Q) = 0
      A(Q,P) = 0
      A(P,P) = C*C*XP(P) + S*S*XQ(Q) - 2*C*S*XQ(P)
      A(Q,Q) = S*S*XP(P) + C*C*XQ(Q) + 2*C*S*XQ(P)
      DO I=1,N
        IF ((I.NE.P).AND.(I.NE.Q)) THEN
      A(I,P) = C*XP(I) - S*XQ(I)
      A(I,Q) = C*XQ(I) + S*XP(I)
      A(P,I) = A(I,P)
      A(Q,I) = A(I,Q)
        ENDIF
      ENDDO
      DO I=1,N
        YP(I) = V(I,P)
        YQ(I) = V(I,Q)
      ENDDO
      DO I=1,N
        V(I,P) = C*YP(I) - S*YQ(I)
        V(I,Q) = S*YP(I) + C*YQ(I)
      ENDDO
      RETURN
      END

      SUBROUTINE MAXPQ(A,N,P,Q,MaxA)
      PARAMETER(MaxR = 4)
      REAL A,MaxA
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER I,J,N,P,Q,Pm,Qm
      MaxA = ABS(A(1,N))
      Pm = 1
      Qm = N
      DO I=1,N
        DO J=(I+1),N
          IF (ABS(A(I,J)).GT.MaxA) THEN
            Pm = I
            Qm = J
            MaxA = ABS(A(I,J))
          ENDIF
        ENDDO
      ENDDO
      P = Pm
      Q = Qm
      RETURN
      END
      
      SUBROUTINE INITIALV(V,N)
      PARAMETER(MaxR = 4)
      REAL V
      DIMENSION V(1:MaxR,1:MaxR)
      INTEGER I,J,N
      DO I=1,N
        DO J=1,N
      IF (I.NE.J) THEN
        V(I,J) = 0
      ELSE
        V(I,J) = 1
      ENDIF
        ENDDO
      ENDDO
      RETURN
      END
      
      REAL FUNCTION RMS(A,N)
      PARAMETER(MaxR = 4)
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER J,N
      REAL Sum
      Sum = 0
      DO J=1,N
        Sum = Sum + A(J,J)
      ENDDO
      RMS = SQRT(Sum/N)
      RETURN
      END
      
      SUBROUTINE SELECTPQ(A,N,P,Q,MaxA,Proc)
      PARAMETER(MaxR = 4)
      REAL A,Apq,MaxA
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER C,I,J,N,P,Q,Ptemp,Qtemp,Proc
      CHARACTER Resp*1
      Manual = 2
      CALL MAXPQ(A,N,P,Q,MaxA)
      Apq = A(P,Q)
      IF (Proc.NE.Manual) RETURN
      CALL MATPRINT(A,N)
      WRITE(9,*)' '
      WRITE(9,*)'     Choose an off diagonal element to "zero-out."'
      WRITE(9,*)' '
      IF (N.EQ.2) THEN
        WRITE(9,*)'Select the row  P = 1,2'
        WRITE(9,*)'   and  column  Q = 1,2'
      ELSEIF (N.EQ.3) THEN
        WRITE(9,*)'Select the row  P = 1,2,3'
        WRITE(9,*)'   and  column  Q = 1,2,3'
      ELSE
        WRITE(9,*)'Select the row  P = 1,2,...,', N
        WRITE(9,*)'   and  column  Q = 1,2,...,', N
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'     ENTER  the  row P = '
      READ(9,*) Ptemp
      IF ((1.LE.Ptemp).AND.(Ptemp.LE.N)) P = Ptemp
      WRITE(9,*)'N = ',N
      WRITE(9,*)'Ptemp = ',Ptemp
      WRITE(9,*)'P = ',P
      WRITE(9,*)'     ENTER  column   Q = '
      READ(9,*) Qtemp
      IF ((1.LE.Qtemp).AND.(Qtemp.LE.N)) Q = Qtemp
      WRITE(9,*)'N = ',N
      WRITE(9,*)'Qtemp = ',Qtemp
      WRITE(9,*)'Q = ',Q
      Apq = A(P,Q)
      WRITE(9,*)' '
      WRITE(9,*)'The element selected was A(',P,',',Q,') =',A(P, Q)
      WRITE(9,*)' '
      PAUSE
      RETURN
      END

      SUBROUTINE OJacob(A,V,N,Epsilon,CountR,Proc)
      PARAMETER(MaxR = 4)
      REAL A,V,C,S,MaxA,Apq,Epsilon
      DIMENSION A(1:MaxR,1:MaxR),V(1:MaxR,1:MaxR)
      INTEGER CountR,N,P,Q,Stat,Done,Working
      INTEGER Proc,Auto,Manual,Observe
      CHARACTER Ans*1
      Done = 0
      Working = 1
      Stat = Working
      Auto = 1
      Manual = 2
      Observe = 3
      CountR = 0
      WHILE(Stat.EQ.Working)
        CALL SELECTPQ(A,N,P,Q,MaxA,Proc)
        Apq = MaxA
        CALL ZEROPQ(A,V,C,S,N,P,Q)
        CountR = CountR+1
        IF (Proc.EQ.Auto) THEN
          WRITE(9,*)'I am computing rotation  # ', CountR
        ENDIF
        IF ((Proc.EQ.Manual).OR.(Proc.EQ.Observe)) THEN
          CALL Routput(P,Q,C,S,Apq)
          CALL MATPRINT(A,N)
        ENDIF
        CALL MAXPQ(A,N,P,Q,MaxA)
        IF (MaxA.LT.Epsilon*RMS(A,N)) Stat = Done
        IF (Proc.EQ.Manual) THEN
          WRITE(9,*)' '
          WRITE(9,*)'Want to zero out another element ? <Y/N>'
          READ(9,*) Ans
          IF ((Ans.EQ.'N').OR.(Ans.EQ.'n')) Stat = Done
        ENDIF
      REPEAT
      RETURN
      END
      
      SUBROUTINE CJacob(A,V,N,Epsilon,CountS,Proc)
      PARAMETER(MaxR = 4, MaxS = 50)
      REAL A,V,C,S,Apq,Epsilon
      DIMENSION A(1:MaxR,1:MaxR),V(1:MaxR,1:MaxR)
      INTEGER CountR,CountS,N,P,Q,Stat,Done,Working
      INTEGER Proc,Auto,Manual,Observe
      EXTERNAL RMS
      CountR = 0
      CountS = 0
      Done = 0
      Working = 1
      Stat = Working
      Auto = 1
      Manual = 2
      Observe = 3
      WHILE ((Stat.EQ.Working).AND.(CountS.LT.MaxS))
        CountS = CountS+1
        T = RMS(A,N)
        Stat = Done
          DO P=1,(N-1)
            DO Q=(P+1),N
              IF ((ABS(A(P,Q))/T).GT.Epsilon) THEN
                Apq = A(P,Q)
                CALL ZEROPQ(A,V,C,S,N,P,Q)
                CountR = CountR+1
                IF (Proc.EQ.Auto) THEN
                  WRITE(9,*)'I am computing sweep # ',
     +            CountS,'  ,  rotation  # ',CountR
                ENDIF
                IF (Proc.EQ.Observe) THEN
                  CALL Routput(P,Q,C,S,Apq)
                  CALL MATPRINT(A,N)
                ENDIF
                Stat = Working
              ENDIF
            ENDDO
          ENDDO
      REPEAT
      RETURN
      END

      SUBROUTINE MATIN(Ach,A,N,InRC)
      PARAMETER(MaxR = 4)
      INTEGER Col,InRC,N,Row
      REAL A
      CHARACTER Ach*1
      DIMENSION A(1:MaxR,1:MaxR)
      DO 20 Row=1,N
      DO 10 Col=1,N
        A(Row,Col)=0
10    CONTINUE
20    CONTINUE
      WRITE(9,*)'GIVE THE ELEMENTS OF THE MATRIX ',Ach
      WRITE(9,*)' '
      IF (InRC.EQ.0) THEN
        DO 40 Row=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF ROW ',Row
          WRITE(9,*)' '
        DO 30 Col=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
30      CONTINUE
40      CONTINUE
      ELSE
        DO 60 Col=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF COLUMN ',Col
          WRITE(9,*)' '
        DO 50 Row=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
50      CONTINUE
60      CONTINUE
      ENDIF
      RETURN
      END
      
      SUBROUTINE PrintResults(A,A1,V,N,Epsilon,MaxA,CountS,Meth)
      PARAMETER(MaxR = 4, MaxS = 50)
      REAL A0,A1,V,Epsilon,MaxA
      DIMENSION A(1:MaxR,1:MaxR), A1(1:MaxR,1:MaxR),V(1:MaxR,1:MaxR)
      INTEGER N,C,R,P,Q,CountS
      INTEGER I,Meth,Cyclic,Original
      EXTERNAL RMS
      Original = 1
      Cyclic = 2
      WRITE(9,*)' '
      WRITE(9,*)'The matrix  A  is:'
      WRITE(9,*)' '
      DO R=1,N
        WRITE(9,999) ( A1(R,C), C=1,N )
      ENDDO
      WRITE(9,*)' '
      WRITE(9,*)'The eigenvalues of  A  are:'
      DO R=1,N
        WRITE(9,*)'X(',R,')  =  ',A(R,R)
      ENDDO
      WRITE(9,*)' '
      WRITE(9,*)'The matrix of eigenvectors  V  is:'
      DO R=1,N
        WRITE(9,999) ( V(R,C), C=1,N )
      ENDDO
      WRITE(9,*)' '
      CALL MAXPQ(A,N,P,Q,MaxA)
      IF ((MaxA.GT.RMS(A,N)).AND.(Meth.EQ.Original)) THEN
        WRITE(9,*)' '
        WRITE(9,*)'The iteration did NOT converge!'
      ENDIF
      IF ((MaxS.LT.CountS).AND.(Meth.EQ.Cyclic)) THEN
        WRITE(9,*)' '
        WRITE(9,*)'The iteration did NOT converge!'
      ENDIF
      PAUSE
      RETURN
999   FORMAT(1X,5F15.7,4X)
      END
      
      SUBROUTINE CHOOSEMETH(Meth)
      INTEGER I,Meth,Cyclic,Original
      Original = 1
      Cyclic = 2
      WRITE(9,*)'Choose a strategy for selecting the off diagonal'
      WRITE(9,*)' '
      WRITE(9,*)'element to annihilate in the construction process.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<1>  Jacobi`s original strategy.'
      WRITE(9,*)' '
      WRITE(9,*)'Select p,q so that |a  | = max |a  |.'
      WRITE(9,*)'                     pq    i<j   ij  '
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<2>  The cyclic Jacobi method.'
      WRITE(9,*)' '
      WRITE(9,*)'Now sweep through the matrix and annihilate'
      WRITE(9,*)' '
      WRITE(9,*)'elements in the strict order'
      WRITE(9,*)' '
      WRITE(9,*)'a  ,a  ,...,a  ;a  ,a  ,...,a  ;...;a     .'
      WRITE(9,*)' 12  13      1N  23  24      2N      N-1,N '
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     SELECT the strategy  < 1 or 2 > ?  '
      READ(9,*) I
      IF ((I.LT.1).OR.(2.LT.I)) I = 1
      IF (I.EQ.1) Meth = Original
      IF (I.EQ.2) Meth = Cyclic
      RETURN
      END
      
      SUBROUTINE MESSAGE(Meth)
      INTEGER I,Meth
      CHARACTER Ans*1
      WRITE(9,*)'            JACOBI`S  METHOD FOR EIGENVECTORS'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     Assume that A is an N by N real symmetric matrix.'
      WRITE(9,*)' '
      WRITE(9,*)'Then A has a full set of eigenvectors'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'V , V ,..., V .  Jacobi`s method of iteration is used'
      WRITE(9,*)' 1   2       N '
      WRITE(9,*)' '
      WRITE(9,*)'to find the eigenvalues and eigenvectors of A. Let'
      WRITE(9,*)' '
      WRITE(9,*)'A = A , and construct a sequence of orthogonal'
      WRITE(9,*)'     1 '
      WRITE(9,*)'                                   T '
      WRITE(9,*)'matrices { R  }  such that  D  =  R  A  R .'
      WRITE(9,*)'            j                j     j  j  j '
      WRITE(9,*)' '
      WRITE(9,*)'{ D  } converges to the diagonal matrix D ,'
      WRITE(9,*)'   j '
      WRITE(9,*)' '
      WRITE(9,*)'of eigenvalues, and the sequence {V  = R R ...R }'
      WRITE(9,*)'                                   j    1 2    j '
      WRITE(9,*)' '
      WRITE(9,*)'converges to the matrix of eigenvectors.'
      WRITE(9,*)' '
      WRITE(9,*)'                     Press the  <ENTER>  key.  '
      READ(9,*) Ans
      CALL CHOOSEMETH(Meth)
      RETURN
      END
      
      SUBROUTINE INPUTS(A,A1,N,Epsilon,InRC)
      PARAMETER(MaxR = 4)
      REAL A,A1,Epsilon
      DIMENSION A(1:MaxR,1:MaxR), A1(1:MaxR,1:MaxR)
      INTEGER InRC,N,C,Row,Col
      CHARACTER Ans*1
      WRITE(9,*)'    We will now proceed with Jacobi`s method'
      WRITE(9,*)' '
      WRITE(9,*)'of iteration to find the set of N eigenvectors'
      WRITE(9,*)' '
      WRITE(9,*)'V , V ,..., V   for the symmetric matrix  A.'
      WRITE(9,*)' 1   2       N '
      WRITE(9,*)' '
      WRITE(9,*)'A must be a symmetric matrix of dimension N by N.'
      WRITE(9,*)' '
      WRITE(9,*)'{N  must be an integer between 2 and 10}'
      WRITE(9,*)' '
      WRITE(9,*)'    ENTER  N  = '
      READ(9,*) N
      IF (N.LT.2) N = 2
      IF (N.GT.10) N = 10
      WRITE(9,*)' '
      WRITE(9,*)'     For each sweep throughout the matrix,'
      WRITE(9,*)' '
      WRITE(9,*)'the computed value T is the [R.M.S.] average'
      WRITE(9,*)' '
      WRITE(9,*)'of the diagonal elements of the matrix  A.'
      WRITE(9,*)' '
      WRITE(9,*)'Give the error criterion for annihilating'
      WRITE(9,*)' '
      WRITE(9,*)'the off diagonal elements'
      WRITE(9,*)' '
      WRITE(9,*)'|d   |  >  T*Epsilon    for all q > p.'
      WRITE(9,*)'  p,q'
      WRITE(9,*)' '
      WRITE(9,*)'      ENTER  Epsilon  = '
      READ(9,*) Epsilon
      IF (Epsilon.LT.0.000000001) Epsilon = 0.000000001
      InRC = 0
      WRITE(9,*)'WANT TO INPUT COLUMNS ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') THEN
        InRC = 1
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE COEFFICIENT MATRIX  A:'
      CALL MATIN('A',A,N,InRC)
      DO 30 Row=1,N
      DO 20 Col=1,N
        A1(Row,Col)=A(Row,Col)
20    CONTINUE
30    CONTINUE
      RETURN
      END
      
      SUBROUTINE PROCESSES(Proc)
      INTEGER I,Proc,Auto,Manual,Observe
      Auto = 1
      Manual = 2
      Observe = 3
      WRITE(9,*)'To what extent do you want to control the program?'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<1> The computer does it all automatically.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<2> The user selects elements to zero-out manually.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<3> Computer selects, but we observe each step.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'SELECT your choice for input  < 1 - 3 > ? '
      READ(9,*) I
      IF ((I.LT.1).OR.(3.LT.I)) I = 3
      IF (I.EQ.1) Proc = Auto
      IF (I.EQ.2) Proc = Manual
      IF (I.EQ.3) Proc = Observe
      RETURN
      END
