      PROGRAM HOUSE
C     ----------------------------------------------------------------
C     Alg11'4.for   FORTRAN program for implementing Algorithm 11.4
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 11.4  (Reduction to Tridiagonal Form).
C     Section   11.4, Eigenvalues of Symmetric Matrices, Page 581
C     ----------------------------------------------------------------
      PARAMETER(MaxR = 5, MaxS = 50)
      CHARACTER Ans*1,Ach*1,Mess*80
      INTEGER CountR,CountS
      INTEGER I,InRC,Inum,K,N
      REAL C,MaxA,Rnum,R,S,T
      REAL W,V,Q
      DIMENSION W(1:MaxR),V(1:MaxR),Q(1:MaxR)
      REAL A,A1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER MatType 
      INTEGER Proc,Auto,Observe
      Auto = 1
      Observe = 2
      CALL MESSAGE(Meth)
      CALL INPUTS(A,A1,N,InRC)
      CALL PROCESSES(Proc)
      CALL HOUSEHOLDER(A,W,V,Q,S,R,C,N,Proc)
      CALL PrintResults(A,A1,N)
      END

      SUBROUTINE MATPRINT(A,K,N)
      PARAMETER(MaxR = 5)
      INTEGER Col,K,N,Row
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      WRITE(9,*) 'The matrix A  is:'
      WRITE(9,*) '            ',K
      DO 10 Row=1,N
        WRITE(9,999) ( A(Row,Col), Col=1,N )
10    CONTINUE
999   FORMAT(1X,5F15.7,4X)
      RETURN
      END
      
      SUBROUTINE StepOutput(W,V,Q,S,R,C,N)
      PARAMETER(MaxR = 5, MaxS = 50)
      REAL W,V,Q,S,R,C
      DIMENSION W(1:MaxR),V(1:MaxR),Q(1:MaxR)
      INTEGER N,J
      WRITE(9,*)' '
      WRITE(9,*)'      Constant S              Constant
     + R              Constant C'
      WRITE(9,*)'     ',S,'                ',
     + R,'                ', C
      WRITE(9,*)' '
      WRITE(9,*)'      Vector W                Vector
     + V                Vector Q'
      DO J=1,N
        WRITE(9,*)'    ',W(J), '                 ',
     + V(J), '                ', Q(J)
      ENDDO
      RETURN
      END

      SUBROUTINE Form_W(W,A,S,R,K,N)
      PARAMETER(MaxR = 5)
      REAL W,A,S,R,Sum
      DIMENSION W(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER J,K,N
      Sum = 0
      DO J=(K+1),N
        Sum = Sum + A(J,K)**2
      ENDDO
      S = SQRT(Sum)
      IF A(K+1,K) < 0 THEN
        S = -S
      ENDIF
      R = SQRT(2*(A(K+1,K) + S)*S)
      DO J=1,K                          !First k elements are 0
        W(J) = 0;
      ENDDO
      W(K+1) = (A(K+1,K) + S)/R
      DO J=(K+2),N
        W(J) = A(J,K)/R
      ENDDO
      RETURN
      END
      
      SUBROUTINE Form_V(V,A,W,K,N)
      PARAMETER(MaxR = 5)
      REAL V,W
      DIMENSION V(1:MaxR),W(1:MaxR)
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER I,J,K,N
      REAL Sum
      DO I=1,K                    !First k elements are 0
        V(I) = 0
      ENDDO
      DO I=(K+1),N
          Sum = 0
          DO J=(K+1),N
            Sum = Sum + A(I,J)*W(J)
          ENDDO
          V(I) = Sum
      ENDDO
      RETURN
      END
      
      SUBROUTINE Form_Q(Q,V,W,C,K,N)
      PARAMETER(MaxR = 5)
      REAL C,V,W,Q
      DIMENSION V(1:MaxR),W(1:MaxR),Q(1:MaxR)
      INTEGER J,K,N
      C = 0
      DO J=(K+1),N 
        C = C + W(J)*V(J)
      ENDDO
      DO J=1,K                       !First k elements are 0}
        Q(J) = 0
      ENDDO
      DO J=(K+1),N
        Q(J) = V(J) - C*W(J)
      ENDDO
      RETURN
      END
      
      SUBROUTINE Form_A(A,Q,W,S,K,N)
      PARAMETER(MaxR = 5)
      REAL A,Q,W,S
      DIMENSION Q(1:MaxR),W(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER I,J,K,N
      DO J=(K+2),N
        A(J,K) = 0
        A(K,J) = 0
      ENDDO
      A(K+1,K) = -S
      A(K,K+1) = -S
      DO J=K,N
        A(J,J) = A(J,J) - 4*Q(J)*W(J)
      ENDDO
      DO I=(K+1),N
          DO J=(I+1),N
        A(I,J) = A(I,J) - 2*W(I)*Q(J) - 2*Q(I)*W(J)
        A(J,I) = A(I,J);
          ENDDO
      ENDDO
      RETURN
      END
            
      SUBROUTINE HOUSEHOLDER(A,W,V,Q,S,R,C,N,Proc)
      PARAMETER(MaxR = 5)
      REAL A,W,V,Q,S,R,C
      DIMENSION W(1:MaxR),V(1:MaxR),Q(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER K,N,Proc,Observe
      Observe = 2
      DO K=1,(N-2)
        CALL Form_W(W,A,S,R,K,N)
        CALL Form_V(V,A,W,K,N)
        CALL Form_Q(Q,V,W,C,K,N)
        CALL Form_A(A,Q,W,S,K,N)
        IF (Proc.EQ.Observe) THEN
          CALL StepOutput(W,V,Q,S,R,C,N)
          CALL MATPRINT(A,K,N)
          WRITE(9,*)' '
          WRITE(9,*)'      Press the <ENTER> key.  '
          PAUSE
        ENDIF
      ENDDO
      RETURN
      END

      SUBROUTINE MATIN(Ach,A,N,InRC)
      PARAMETER(MaxR = 5)
      INTEGER Col,InRC,N,Row
      REAL A
      CHARACTER Ach*1
      DIMENSION A(1:MaxR,1:MaxR)
      DO 20 Row=1,N
      DO 10 Col=1,N
        A(Row,Col)=0
10    CONTINUE
20    CONTINUE
      WRITE(9,*)'GIVE THE ELEMENTS OF THE MATRIX ',Ach
      WRITE(9,*)' '
      IF (InRC.EQ.0) THEN
        DO 40 Row=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF ROW ',Row
          WRITE(9,*)' '
        DO 30 Col=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
30      CONTINUE
40      CONTINUE
      ELSE
        DO 60 Col=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF COLUMN ',Col
          WRITE(9,*)' '
        DO 50 Row=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
50      CONTINUE
60      CONTINUE
      ENDIF
      RETURN
      END
      
      SUBROUTINE MESSAGE(Meth)
      INTEGER Meth
      WRITE(9,*)'            HOUSEHOLDER`S  METHOD'
      WRITE(9,*)' '
      WRITE(9,*)'  Assume that A is an N by N real symmetric matrix.'
      WRITE(9,*)' '
      WRITE(9,*)'Then A is similar to a tridiagonal matrix.'
      WRITE(9,*)' '
      WRITE(9,*)'Starting with  A  = A, Householder`s method will'
      WRITE(9,*)'                0 '
      WRITE(9,*)' '
      WRITE(9,*)'construct a sequence of orthogonal symmetric matrices'
      WRITE(9,*)' '
      WRITE(9,*)'{P } such that A  = P  A    P  ,'
      WRITE(9,*)'  k             k    k  k-1  k  '
      WRITE(9,*)' '
      WRITE(9,*)'           for  k = 1,2,...,N-2.'
      WRITE(9,*)' '
      WRITE(9,*)'Then  A    is the desired tridiagonal matrix.'
      WRITE(9,*)'       N-2 '
      WRITE(9,*)' '
      PAUSE
      RETURN
      END
      
      SUBROUTINE INPUTS(A,A1,N,InRC)
      PARAMETER(MaxR = 5)
      REAL A,A1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER N,InRC,Row,Col
      WRITE(9,*)'    We will now proceed with Householder`s'
      WRITE(9,*)' '
      WRITE(9,*)'method of iteration to find the tridiagonal'
      WRITE(9,*)' '
      WRITE(9,*)'matrix  A    that is similar to  A.'
      WRITE(9,*)'         N-2 '
      WRITE(9,*)' '
      WRITE(9,*)'A  must be a symmetric N by N matrix.'
      WRITE(9,*)' '
      WRITE(9,*)'{N  must be an integer between 2 and 10}'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  N  = '
      READ(9,*) N
      IF (N.LT.2) N = 2
      IF (N.GT.10) N = 10
      InRC = 0
      WRITE(9,*)'WANT TO INPUT COLUMNS ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') THEN
        InRC = 1
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE COEFFICIENT MATRIX  A:'
      CALL MATIN('A',A,N,InRC)
      DO 30 Row=1,N
      DO 20 Col=1,N
        A1(Row,Col)=A(Row,Col)
20    CONTINUE
30    CONTINUE
      RETURN
      END
      
      SUBROUTINE PROCESSES(Proc)
      INTEGER I,Proc,Auto,Observe
      Auto = 1
      Observe = 2
      WRITE(9,*)'To what extent do you want to control the program?'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<1> The computer does it all automatically.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<3> Computer selects, but we observe each step.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'SELECT your choice for input  <1 or 2> ? '
      READ(9,*) I
      IF ((I.LT.1).OR.(3.LT.I)) I = 3
      IF (I.EQ.1) Proc = Auto
      IF (I.EQ.2) Proc = Observe
      RETURN
      END

      SUBROUTINE PrintResults(A,A1,N)
      PARAMETER(MaxR = 5)
      REAL A,A1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER N,R,C
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'The matrix  A  is:'
      WRITE(9,*)' '
      DO 10 R=1,N
        WRITE(9,999) ( A1(R,C), C=1,N )
      ENDDO
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'The similar tridiagonal matrix is:'
      DO 10 R=1,N
        WRITE(9,999) ( A(R,C), C=1,N )
      ENDDO
      PAUSE
      RETURN
999   FORMAT(1X,5F15.7,4X)
      END
