      PROGRAM QLALGOR
C     ----------------------------------------------------------------
C     Alg11'5.for   FORTRAN program for implementing Algorithm 11.5
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 11.5 (The QL Method with Shifts).
C     Section   11.4, Eigenvalues of Symmetric Matrices, Page 587
C     ----------------------------------------------------------------
      PARAMETER(MaxR = 10,MaxS = 50)
      CHARACTER Ans*1,Ach*1,Mess*80
      INTEGER CountR,CountS,I,InRC,Inum,K,N
      REAL MaxA,Rnum,T
      REAL C,D,E,P,R,S,Q
      DIMENSION C(1:MaxR),D(1:MaxR),E(1:MaxR)
      DIMENSION P(1:MaxR),R(1:MaxR),S(1:MaxR),Q(1:MaxR)
      DIMENSION YP(1:MaxR),YQ(1:MaxR)
      REAL A,A1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      CALL MESSAGE(InRC,Mtype)
      CALL INPUTS(A,A1,N,InRC)
      CALL PROCESSES
      CALL QL_ITERATION(A,D,N)
      CALL PrintResults(A,A1,D,N)
      END

      SUBROUTINE MATPRINT(A,K,N)
      PARAMETER(MaxR=10)
      INTEGER Col,K,N,Row
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      WRITE(9,*) 'The matrix A  is:'
      WRITE(9,*) '            ',K
      DO 10 Row=1,N
        WRITE(9,999) ( A(Row,Col), Col=1,N )
10    CONTINUE
999   FORMAT(1X,5F15.7,4X)
      RETURN
      END

      SUBROUTINE A_TO_TRI(D,E,A,N)
      REAL D,E,A
      PARAMETER(MaxR = 10)
      DIMENSION D(1:MaxR),E(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER N,J
      DO J=1,N
        D(J) = A(J,J)
      ENDDO
      DO J=1,(N-1)
        E(J) = A(J,J+1)
      ENDDO
      RETURN
      END

      SUBROUTINE TRI_TO_A(D,E,A,N)
      REAL D,E,A
      PARAMETER(MaxR = 10)
      DIMENSION C(1:MaxR),D(1:MaxR),E(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER N,J
      DO J=1,N
        A(J,J) = D(J)
      ENDDO
      DO J=1,(N-1)
        A(J,J+1) = E(J)
        A(J+1,J) = E(J)
      ENDDO
      RETURN
      END

      SUBROUTINE Find_Shift(D,E,SH,R1,R2,M)
      PARAMETER(MaxR = 10)
      REAL D,E,SH,B1,C1,D1,R1,R2
      DIMENSION D(1:MaxR),E(1:MaxR)
      INTEGER M
      B1 = -(D(M+1) + D(M))
      C1 = D(M+1)*D(M) - E(M)*E(M)
      D1 = SQRT(ABS(B1*B1 - 4*C1))
      IF (B1.GT.0) THEN
          R1 = (-B1 - D1)/2;
          R2 = 2*C1/(-B1 - D1)
      ELSE
          R1 = (-B1 + D1)/2
          R2 = 2*C1/(-B1 + D1)
      ENDIF
      SH = R1
      IF (ABS(D(M) - R2).LT.ABS(D(M) - R1)) THEN
        SH = R2
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'the shift is  ',SH
      WRITE(9,*)' '
      PAUSE
      RETURN
      END

      SUBROUTINE Form_L(C,D,E,P,S,Q,N,M)
      PARAMETER(MaxR = 10)
      REAL C,D,E,P,S,Q
      DIMENSION C(1:MaxR),D(1:MaxR),E(1:MaxR)
      DIMENSION P(1:MaxR),S(1:MaxR),Q(1:MaxR)
      INTEGER N,M,J
      REAL P_J1,Q_J
      DO J=1,N
        P(J) = D(J)
      ENDDO
      DO J=1,(N-1)
        Q(J) = E(J)
      ENDDO
      P_J1 = P(N)                    !P_J1 old value of P(J+1)
      Q_J = Q(N-1)                   !Q_J old value of Q(J)
      DO J=(N-1),M,-1
          P(J+1) = SQRT(P_J1**2 + Q(J)**2)
          C(J) = P_J1/P(J+1)
          S(J) = Q(J)/P(J+1)
          Q(J) = C(J)*Q_J + S(J)*D(J)
          P_J1 = C(J)*D(J) - S(J)*Q_J
          IF (J.GT.M) THEN
            ! R(J-1) = S(J)*Q(J-1)  dead code
            Q_J = C(J)*Q(J-1)
          ENDIF
      ENDDO
      P(M) = P_J1
      RETURN
      END

      SUBROUTINE Form_A(C,D,E,P,S,Q,N,M)
      PARAMETER(MaxR = 10)
      REAL C,D,E,P,S,Q
      DIMENSION C(1:MaxR),D(1:MaxR),E(1:MaxR)
      DIMENSION P(1:MaxR),S(1:MaxR),Q(1:MaxR)
      INTEGER N,M,J
      D(N) = S(N-1)*Q(N-1) + C(N-1)*P(N)
      E(N-1) = S(N-1)*P(N-1)
      DO J=(N-2),M,-1
        D(J+1) = S(J)*Q(J) + C(J)*C(J+1)*P(J+1)
        E(J) = S(J)*P(J)
      ENDDO
      D(M) = C(M)*P(M)
      RETURN
      END

      SUBROUTINE QL_ITERATION(A,D,N)
      PARAMETER(DELTA = 0.00000001)
      PARAMETER(MaxR = 10)
      REAL A,C,D,E,P,R,S,Q
      DIMENSION C(1:MaxR),D(1:MaxR),E(1:MaxR)
      DIMENSION P(1:MaxR),R(1:MaxR),S(1:MaxR),Q(1:MaxR)
      DIMENSION A(1:MaxR,1:MaxR)
      INTEGER Cond,J,M,N,Proc,Observe
      REAL SH,SHIFT,R1,R2
      Observe = 1
      IF (Proc.EQ.Observe) THEN
        CALL Aoutput(A,0,N)
      ENDIF
      CALL A_TO_TRI(D,E,A,N)
      SHIFT = 0
      M = 1
      WHILE (M.LE.(N-2))
        K = 1
        Cond = 0
        WHILE ((K.LT.50).AND.(Cond.EQ.0))
          CALL Find_Shift(D,E,SH,R1,R2,M)
          IF (ABS(E(M)).GT.DELTA) THEN
            SHIFT = SHIFT + SH
            DO J=M,N
              D(J) = D(J) - SH
            ENDDO
          ELSE
            WRITE(9,*)' '
            WRITE(9,*)'the eigenvalue is '
            D(M) = D(M) + SHIFT;
            WRITE(9,*) D(M)
            WRITE(9,*)' '
            WRITE(9,*)'     Press the <ENTER> key.  '
            PAUSE
            M = M+1
            Cond = 1
          ENDIF
          CALL Form_L(C,D,E,P,S,Q,N,M)
          CALL Form_A(C,D,E,P,S,Q,N,M)
          IF (Proc.EQ.Observe) THEN
            CALL TRI_TO_A(D,E,A,N)
            CALL Aoutput(A,K,N)
            WRITE(9,*)' '
            WRITE(9,*)'     Press the <ENTER> key.  '
            PAUSE
          ENDIF
          K = K+1
        REPEAT
      REPEAT
      CALL Find_Shift(D,E,SH,R1,R2,M)
      D(N-1) = R1 + SHIFT
      D(N) = R2 + SHIFT
      DO J=1,(N-1)
        E(J) = 0
      ENDDO
      CALL TRI_TO_A(D,E,A,N)
      RETURN
      END

      SUBROUTINE MATIN(Ach,A,N,InRC)
      PARAMETER(MaxR = 10)
      INTEGER Col,InRC,N,Row
      REAL A
      CHARACTER Ach*1
      DIMENSION A(1:MaxR,1:MaxR)
      DO 20 Row=1,N
      DO 10 Col=1,N
        A(Row,Col)=0
10    CONTINUE
20    CONTINUE
      WRITE(9,*)'GIVE THE ELEMENTS OF THE MATRIX ',Ach
      WRITE(9,*)' '
      IF (InRC.EQ.0) THEN
        DO 40 Row=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF ROW ',Row
          WRITE(9,*)' '
        DO 30 Col=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
30      CONTINUE
40      CONTINUE
      ELSE
        DO 60 Col=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF COLUMN ',Col
          WRITE(9,*)' '
        DO 50 Row=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
50      CONTINUE
60      CONTINUE
      ENDIF
      RETURN
      END
      
      SUBROUTINE PrintResults(A,A1,D,N)
      PARAMETER(MaxR = 10)
      REAL A,A1,D
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      DIMENSION D(1:MaxR)
      INTEGER J,N,R,C
      WRITE(9,*)' '
      WRITE(9,*)'The matrix  A  is:'
      WRITE(9,*)' '
      DO 10 R=1,N
        WRITE(9,999) ( A1(R,C), C=1,N )
      ENDDO
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'The similar diagonal matrix is:'
      DO 10 R=1,N
        WRITE(9,999) ( A(R,C), C=1,N )
      ENDDO
        WRITE(9,*)' '
      WRITE(9,*)' '
      DO J=1,N 
        WRITE(9,*)'the ',J,'th eigenvalue is ',D(J)
      ENDDO
      WRITE(9,*)' '
      PAUSE
      RETURN
999   FORMAT(1X,5F15.7,4X)
      END
      
      SUBROUTINE MESSAGE(Meth)
      WRITE(9,*)'           THE QL ALGORITHM'
      WRITE(9,*)' '
      WRITE(9,*)'    Assume that A is real, symmetric'
      WRITE(9,*)' '
      WRITE(9,*)'and tridiagonal. Start with A  = A.'
      WRITE(9,*)'                             1     '
      WRITE(9,*)' '
      WRITE(9,*)'The QL method constructs a sequence '
      WRITE(9,*)' '
      WRITE(9,*)'of orthogonal matrices {Q } such that'
      WRITE(9,*)'                         k           '
      WRITE(9,*)' '
      WRITE(9,*)'A  = Q L  where L  is lower-triangular.'
      WRITE(9,*)' k    k k        k'
      WRITE(9,*)'Then A    is generated by'
      WRITE(9,*)'      k+1  '
      WRITE(9,*)' '
      WRITE(9,*)'A     =  Q  A  Q    for k = 1,2, ... .'
      WRITE(9,*)' k+1      k  k  k'
      WRITE(9,*)' '
      WRITE(9,*)'The success of the method depends on'
      WRITE(9,*)' '
      WRITE(9,*)'the result that   lim  A  = D,  where'
      WRITE(9,*)'                 k->oo  k            '
      WRITE(9,*)' '
      WRITE(9,*)'D is a diagonal matrix with the same '
      WRITE(9,*)'eigenvalues as the original matrix A.'
      WRITE(9,*)' '
      PAUSE
      RETURN
      END
      
      SUBROUTINE INPUTS(A,A1,N,InRC)
      PARAMETER(MaxR = 10)
      REAL A,A1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      INTEGER N,InRC,Row,Col
      WRITE(9,*)'     We will now proceed with the  QL'
      WRITE(9,*)' '
      WRITE(9,*)'method of iteration to reduce the'
      WRITE(9,*)' '
      WRITE(9,*)'tridiagonal matrix  A  to diagonal form.'
      WRITE(9,*)' '
      WRITE(9,*)'A  must be a tridiagonal matrix of dimension  N by N.'
      WRITE(9,*)' '
      WRITE(9,*)'{N  must be an integer between 2 and 10}'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER  N  = '
      READ(9,*) N
      IF (N.LT.2) N = 2;
      IF (N.GT.10) N = 10
      InRC = 0
      WRITE(9,*)'WANT TO INPUT COLUMNS ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') THEN
        InRC = 1
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE COEFFICIENT MATRIX  A:'
      CALL MATIN('A',A,N,InRC)
      DO 30 Row=1,N
      DO 20 Col=1,N
        A1(Row,Col)=A(Row,Col)
20    CONTINUE
30    CONTINUE
      RETURN
      END
      
      SUBROUTINE PROCESSES(Proc)
      INTEGER I,Proc,Auto,Manual,Observe
      Auto = 0
      Manual = 1
      Observe =2
      WRITE(9,*)'To what extent do you want to control the program?'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<1> The computer does it all automatically.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'<2> Computer selects, but we observe each step.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'SELECT your choice for input  < 1 - 2 > ? '
      READ(9,*) I
      IF ((I.LT.1).OR.(3.LT.I)) I = 3
      IF (I.EQ.1) Proc = Auto
      IF (I.EQ.2) Proc = Observe
      IF (I.EQ.3) Proc = Manual
      RETURN
      END
