      PROGRAM FIXPOINT
C     ----------------------------------------------------------------
C     Alg2'1.for   FORTRAN program for implementing Algorithm 2.1
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 2.1  (Fixed Point Iteration).
C     Section   2.1, Iteration for Solving  x = g(x), Page  51
C     ----------------------------------------------------------------
      PARAMETER(Tol=0.00000001,Max=500)
      INTEGER Cond,K
      REAL P0,Pnew,Pterm
      EXTERNAL G
10    CALL GETPOINT(P0,Pterm)
      CALL ITERATE(G,Pterm,Max,Tol,Pnew,Cond,K)
      CALL RESULTS(P0,Max,Tol,Pnew,Pterm,Cond,K)
      WRITE(9,*) '     WANT TO DO ANOTHER ONE <Y/N> ? '
      READ(9,'(A)') ANS
      IF(ANS .EQ. 'Y' .OR. ANS .EQ. 'y') GOTO 10
      STOP
      END

      REAL FUNCTION G(X)
      REAL X
        G=0.9 + X - 0.4*X*X
      RETURN
      END

      SUBROUTINE PRINTFN
        WRITE(9,*)'G(X) = 0.9 + X - 0.4*X*X'
      RETURN
      END

      SUBROUTINE ITERATE(G,Pterm,Max,Tol,Pnew,Cond,K)
      PARAMETER(Big=1E10,Small=1E-20)
      INTEGER Cond,K,Max
      REAL Pnew,Pterm,Tol
      REAL Dx,Dg,Pold,RelErr,Slope
      K=0
      RelErr=1
      Pnew=G(Pterm)
      WHILE (RelErr.GE.Tol).AND.(K.LE.Max)
     &.AND.(ABS(Pnew).LT.Big)
        Pold=Pterm
        Pterm=Pnew
        Pnew=G(Pterm)
        Dg=Pnew-Pterm
        RelErr=ABS(Dg)/(ABS(Pnew)+Small)
        K=K+1
        WRITE(9,1000) K,Pnew
      REPEAT
      IF (Dg.EQ.0) THEN
        Slope=0
      ELSE
        Dx=Pterm-Pold
        IF (Dx.NE.0) THEN
          Slope=Dg/Dx
        ELSE
          Slope=6.023E23
        ENDIF
      ENDIF
      IF (ABS(Slope).LT.1) THEN
        Cond=1
        IF (Slope.LT.0) Cond=2
      ELSE
        Cond=3
        IF (Slope.LT.0) Cond=4
      ENDIF
      IF (RelErr.LT.Tol) THEN
        IF ((Cond.EQ.3).OR.(Cond.EQ.4)) Cond=5
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(I2,4X,F15.7)
      END

      SUBROUTINE XTERATE(G,Pterm,Max,Tol,Pnew,Cond,K)
      PARAMETER(Big=1E10,Small=1E-20)
      INTEGER Cond,K,Max
      REAL Pnew,Pterm,Tol
      REAL Dx,Dg,Pold,RelErr,Slope
      K=0
      RelErr=1
      Pnew=G(Pterm)
10    IF ((RelErr.GE.Tol).AND.(K.LE.Max)) THEN
        Pold=Pterm
        Pterm=Pnew
        Pnew=G(Pterm)
        Dg=Pnew-Pterm
        RelErr=ABS(Dg)/(ABS(Pnew)+Small)
        K=K+1
        IF ((Pnew.LT.(-Big)).OR.(Big.LT.Pnew)) GOTO 20
        WRITE(9,1000) K,Pnew
        GOTO 10
      ENDIF
20    CONTINUE  !Precludes Overflow
      IF (Dg.EQ.0) THEN
        Slope=0
      ELSE
        Dx=Pterm-Pold
        IF (Dx.NE.0) THEN
          Slope=Dg/Dx
        ELSE
          Slope=6.023E23
        ENDIF
      ENDIF
      IF (ABS(Slope).LT.1) THEN
        Cond=1
        IF (Slope.LT.0) Cond=2
      ELSE
        Cond=3
        IF (Slope.LT.0) Cond=4
      ENDIF
      IF (RelErr.LT.Tol) THEN
        IF ((Cond.EQ.3).OR.(Cond.EQ.4)) Cond=5
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(I2,4X,F15.7)
      END

      SUBROUTINE GETPOINT(P0,Pterm)
      REAL P0,Pterm
      DO 10 I=1,17
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'FIXED POINT ITERATION IS USED TO FIND'
      WRITE(9,*)' '
      WRITE(9,*)'A FIXED POINT OF THE FUNCTION'
      WRITE(9,*)' '
      CALL PRINTFN
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE STARTING VALUE P0 = '
      READ(9,*)  P0
      Pterm=P0
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULTS(P0,Max,Tol,Pnew,Pterm,Cond,K)
      PARAMETER(Small=1E-20)
      INTEGER Cond,I,K,Max
      REAL Delta,P0,Pnew,Pterm,RelErr,Tol
      DO 10 I=1,12
        WRITE(9,*)' '
10     CONTINUE
      WRITE(9,*)'FIXED POINT ITERATION WAS USED TO FIND A FIXED POINT OF
     +'
      WRITE(9,*)' '
      CALL PRINTFN
      WRITE(9,*)' '
      WRITE(9,*)'THE STARTING POINT WAS  P(0) =',P0
      WRITE(9,*)' '
      Delta=ABS(Pnew-Pterm)
      RelErr=Delta/(ABS(Pnew)+Small)
      IF (RelErr.LE.Tol) THEN
        WRITE(9,*)'AFTER ',K,' ITERATIONS THE FIXED POINT WAS FOUND'
        WRITE(9,*)' '
        WRITE(9,*)'    P  =',Pnew
        WRITE(9,*)' '
        WRITE(9,*)'ITERATION WAS SUCCESSFUL.'
      ELSE
        WRITE(9,*)'THE LOCATION OF THE FIXED POINT IS IN DOUBT.'
        WRITE(9,*)' '
        WRITE(9,*)'THE APPROXIMATION AFTER ',K,' ITERATIONS IS'
        WRITE(9,*)' '
        WRITE(9,*)'P(',K,') =',Pnew
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'        ',Pnew,' = G(',Pterm,'  )'
      WRITE(9,*)' '
      WRITE(9,*)'CONSECUTIVE ITERATES ARE WITHIN',Delta
      WRITE(9,*)' '
      IF (K.GE.Max) THEN
        WRITE(9,*)'THE MAXIMUM NUMBER OF ITERATIONS WAS EXCEEDED'
        WRITE(9,*)' '
      ENDIF
      IF (Cond.EQ.1) THEN
        WRITE(9,*)'THE SEQUENCE EXHIBITS MONOTONE CONVERGENCE.'
      ELSEIF (Cond.EQ.2) THEN
        WRITE(9,*)'THE SEQUENCE EXHIBITS OSCILLATING CONVERGENCE.'
      ELSEIF (Cond.EQ.3) THEN
        WRITE(9,*)'THE SEQUENCE EXHIBITS MONOTONE DIVERGENCE.'
      ELSEIF (Cond.EQ.4) THEN
        WRITE(9,*)'THE SEQUENCE EXHIBITS OSCILLATING DIVERGENCE.'
      ENDIF
      DO 20 I=1,1
        WRITE(9,*)' '
20    CONTINUE
      RETURN
      END
