      PROGRAM NEW2DIM
C     ----------------------------------------------------------------
C     Alg2'10.for   FORTRAN program for implementing Algorithm 2.10
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 2.10 (Newton-Raphson Method in Two Dimensions)
C     Section   2.7, Newton's Method for Systems, Page 116
C     ----------------------------------------------------------------
      PARAMETER(Max=99,Delta=1E-6,Epsilon=1E-6)
      INTEGER Cond,K
      REAL D,Pstart,P0,P1,F1
      DIMENSION F1(1:2),Pstart(1:2),P0(1:2),P1(1:2)
      DIMENSION D(1:2,1:2)
      CHARACTER Ans*1
10    CALL INPUTS(Pstart,P0)
      CALL NEWTON(P0,P1,F1,D,Cond,K,Delta,Epsilon,Max)
      CALL RESULTS(Pstart,P0,P1,F1,Cond,K)
      WRITE(9,*)' '
      WRITE(9,*)'Want to try a different starting point ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') GOTO 10
      END

      SUBROUTINE FUN(P,F)
      REAL F,P,X,Y
      DIMENSION F(1:2),P(1:2)
      X=P(1)
      Y=P(2)
      F(1)=X*X-2*X-Y+0.5
      F(2)=X*X+4*Y*Y-4
      RETURN
      END

      SUBROUTINE JACOBIAN(P,D)
      REAL D,P,X,Y
      DIMENSION D(1:2,1:2),P(1:2)
      X=P(1)
      Y=P(2)
      D(1,1)=2*X-2
      D(1,2)=-1
      D(2,1)=2*X
      D(2,2)=8*Y
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'     0 = F (X,Y) = X*X - 2*X - Y + 0.5'
      WRITE(9,*)'          1'
      WRITE(9,*)' '
      WRITE(9,*)'     0 = F (X,Y) = X*X + 4*Y*Y - 4'
      WRITE(9,*)'          2'
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE NEWTON(P0,P1,F1,D,Cond,K,Delta,Epsilon,Max)
      INTEGER Cond,K,Max
      REAL D,Delta,Det,DP,Epsilon,F0,F1,FnZero,P0,P1,Small,RelErr
      DIMENSION DP(1:2),F0(1:2),F1(1:2),Pstart(1:2)
      DIMENSION P0(1:2),P1(1:2),D(1:2,1:2)
      Small=Delta
      K=0
      Cond=0
      CALL FUN(P0,F0)
      P1(1)=P0(1)
      P1(2)=P0(2)
      F1(1)=F0(1)
      F1(2)=F0(2)
      WRITE(9,1000) K,( P1(J), J=1,2 )
      WHILE (K.LT.Max.AND.Cond.EQ.0)
        P0(1)=P1(1)
        P0(2)=P1(2)
        F0(1)=F1(1)
        F0(2)=F1(2)
        K=K+1
        CALL JACOBIAN(P0,D)
        Det=D(1,1)*D(2,2)-D(1,2)*D(2,1)
        IF (Det.EQ.0) THEN
          DP(1)=0
          DP(2)=0
          Cond=1
        ELSE
          DP(1)=(F0(1)*D(2,2)-F0(2)*D(1,2))/Det
          DP(2)=(F0(2)*D(1,1)-F0(1)*D(2,1))/Det
        ENDIF
        P1(1)=P0(1)-DP(1)
        P1(2)=P0(2)-DP(2)
        CALL FUN(P1,F1)
        RelErr=(ABS(DP(1))+ABS(DP(2)))/(ABS(P1(1))+ABS(P1(2))+Small)
        FnZero=ABS(F1(1))+ABS(F1(2))
        IF (RelErr.LT.Delta.AND.FnZero.LT.Epsilon) THEN
          IF (Cond.NE.1) Cond=2
        ENDIF
        WRITE(9,1000) K, ( P1(J), J=1,2 )
      REPEAT
1000  FORMAT(I2,4X,5F15.7)
      PAUSE
      RETURN
      END

      SUBROUTINE XEWTON(P0,P1,F1,D,Cond,K,Delta,Epsilon,Max)
      INTEGER Cond,K,Max
      REAL D,Delta,Det,DP,Epsilon,F0,F1,FnZero,P0,P1,Small,RelErr
      DIMENSION DP(1:2),F0(1:2),F1(1:2),Pstart(1:2)
      DIMENSION P0(1:2),P1(1:2),D(1:2,1:2)
      Small=Delta
      K=0
      Cond=0
      CALL FUN(P0,F0)
      P1(1)=P0(1)
      P1(2)=P0(2)
      F1(1)=F0(1)
      F1(2)=F0(2)
      WRITE(9,1000) K,( P1(J), J=1,2 )
10    IF (K.LT.Max.AND.Cond.EQ.0) THEN
        P0(1)=P1(1)
        P0(2)=P1(2)
        F0(1)=F1(1)
        F0(2)=F1(2)
        K=K+1
        CALL JACOBIAN(P0,D)
        Det=D(1,1)*D(2,2)-D(1,2)*D(2,1)
        IF (Det.EQ.0) THEN
          DP(1)=0
          DP(2)=0
          Cond=1
        ELSE
          DP(1)=(F0(1)*D(2,2)-F0(2)*D(1,2))/Det
          DP(2)=(F0(2)*D(1,1)-F0(1)*D(2,1))/Det
        ENDIF
        P1(1)=P0(1)-DP(1)
        P1(2)=P0(2)-DP(2)
        CALL FUN(P1,F1)
        RelErr=(ABS(DP(1))+ABS(DP(2)))/(ABS(P1(1))+ABS(P1(2))+Small)
        FnZero=ABS(F1(1))+ABS(F1(2))
        IF (RelErr.LT.Delta.AND.FnZero.LT.Epsilon) THEN
          IF (Cond.NE.1) Cond=2
        ENDIF
        WRITE(9,1000) K, ( P1(J), J=1,2 )
        GOTO 10
      ENDIF
1000  FORMAT(I2,4X,5F15.7)
      PAUSE
      RETURN
      END

      SUBROUTINE INPUTS(Pstart,P0)
      REAL Pstart,P0
      DIMENSION Pstart(1:2),P0(1:2)
      DO 10 I=1,10
      WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'     Newton-Raphson iteration is used to find a'
      WRITE(9,*)' '
      WRITE(9,*)'     solution point (P,Q) of the non-linear system'
      WRITE(9,*)' '
      WRITE(9,*)'     0 = F (X,Y)'
      WRITE(9,*)'          1'
      WRITE(9,*)' '
      WRITE(9,*)'     0 = F (X,Y)   The current system of equations is:'
      WRITE(9,*)'          2'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)'     ENTER the initial starting point (P ,Q )'
      WRITE(9,*)'                                        0  0 '
      WRITE(9,*)' '
      WRITE(9,*)'     P(0) = '
      READ(9,*) P0(1)
      Pstart(1)=P0(1)
      WRITE(9,*)' '
      WRITE(9,*)'     Q(0) = '
      READ(9,*) P0(2)
      Pstart(2)=P0(2)
      RETURN
      END

      SUBROUTINE RESULTS(Pstart,P0,P1,F1,Cond,K)
      INTEGER Cond,K
      REAL F1,P0,P1
      DIMENSION F1(1:2),Pstart(1:2),P0(1:2),P1(1:2)
      K=Count
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'Newton-Raphson iteration was used to solve the non-line
     +ar system'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)'starting with  P =',Pstart(1),'   and   Q =',Pstart(2)
      WRITE(9,*)'                0                             0'
      WRITE(9,*)' '
      WRITE(9,*)'It took ',K,' iterations to compute the solution point
     +(P,Q).'
      WRITE(9,*)' '
      IF (Cond.EQ.0) THEN
        WRITE(9,*)'However, the maximum number of iterations was exceede
     +d.'
      ELSEIF (Cond.EQ.1) THEN
        WRITE(9,*)'However, division by zero was encountered.'
      ELSEIF (Cond.EQ.2) THEN
        WRITE(9,*)'The solution is within the desired tolerances.'
      ENDIF
      WRITE(9,*)' '
      IF (Cond.EQ.0.OR.Cond.EQ.1) THEN
        WRITE(9,*)'         P(',K,') =',P1(1),'  ,  Q(',K,') =',P1(2)
      ELSEIF (Cond.EQ.2) THEN
        WRITE(9,*)'            P  =',P1(1),'  ,     Q  =',P1(2)
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'          |DP| =',ABS(P1(1)-P0(1)),'  ,   |DQ| =',ABS(P
     +1(2)-P0(2))
      WRITE(9,*)' '
      WRITE(9,*)'       F (P,Q) =',F1(1)
      WRITE(9,*)'        1'
      WRITE(9,*)'       F (P,Q) =',F1(2)
      WRITE(9,*)'        2'
      RETURN
      END
