      PROGRAM MULMETH
C     ----------------------------------------------------------------
C     Alg2'8.for   FORTRAN program for implementing Algorithm 2.8
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 2.8 (Muller's Method).
C     Section   2.5, Aitken's Process, Steffensen's & Muller's Methods
C     Page 97
C     ----------------------------------------------------------------
      PARAMETER(Delta=5E-6,Epsilon=5E-6,Max=99)
      INTEGER Cond,K
      REAL P0,P1,P2,P3,Q0,Q1,Q2,Z
      CHARACTER ANS*1
      EXTERNAL F
10    CALL INPUTS(P0,P1,P2)
      Q0=P0
      Q1=P1
      Q2=P2
      CALL MULLER(F,Q0,Q1,Q2,Delta,Epsilon,Max,P3,Z,K,Cond)
      CALL RESULT(P0,P1,P2,P3,Z,K,Cond)
      WRITE(9,*)
      WRITE(9,*) 'WANT TO TRY NEW STARTING VALUES ? <Y/N> '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
      REAL X
      F=X*X*X-3*X+2
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)'F(X)  =  X*X*X-3*X+2'
      RETURN
      END

      SUBROUTINE MULLER(F,P0,P1,P2,Delta,Epsilon,Max,P3,Z,K,Cond)
      PARAMETER(Small=1E-20)
      INTEGER Cond,K,Max
      REAL A,B,C,Det,Delta,Disc,Epsilon,H0,H1,E0,E1
      REAL P0,P1,P2,P3,Y0,Y1,Y2,RelErr,U,V,Z
      EXTERNAL F
      K=0
      Cond=0
      Y0=F(P0);  Y1=F(P1);  Y2=F(P2)
      WHILE ((K.LT.Max).AND.(Cond.EQ.0))
        H0=P0-P2
        H1=P1-P2
        C=Y2
        E0=Y0-C
        E1=Y1-C
        Det=H0*H1*(H0-H1)
        A=(E0*H1-H0*E1)/Det
        B=(H0*H0*E1-H1*H1*E0)/Det
        IF ((B*B).GT.(4*A*C)) THEN
          Disc=SQRT(B*B-4*A*C)
        ELSE
          DISC=0
        ENDIF
        IF (B.LT.0) Disc=-Disc
        Z=-2*C/(B+Disc)
        P3=P2+Z
        IF (ABS(P3-P1).LT.ABS(P3-P0)) THEN
          U=P1;  P1=P0;  P0=U
          V=Y1;  Y1=Y0;  Y0=V
        ENDIF
        IF (ABS(P3-P2).LT.ABS(P3-P1)) THEN
          U=P2;  P2=P1;  P1=U
          V=Y2;  Y2=Y1;  Y1=V
        ENDIF
        P2=P3;  Y2=F(P2)
        RelErr=ABS(Z)/(ABS(P2)+Small)
        IF (RelErr.LT.Delta).AND.(ABS(Y2).LT.Epsilon) Cond=1
        K=K+1
        WRITE(9,1000) K,P2,Y2
      REPEAT
      PAUSE
      RETURN
1000  FORMAT(I2,4X,F15.7,4X,F15.7)
      END

      SUBROUTINE XULLER(F,P0,P1,P2,Delta,Epsilon,Max,P3,Z,K,Cond)
      PARAMETER(Small=1E-20)
      INTEGER Cond,K,Max
      REAL A,B,C,Det,Delta,Disc,Epsilon,H0,H1,E0,E1
      REAL P0,P1,P2,P3,Y0,Y1,Y2,RelErr,U,V,Z
      EXTERNAL F
      K=0
      Cond=0
      Y0=F(P0)
      Y1=F(P1)
      Y2=F(P2)
10    IF ((K.LT.Max).AND.(Cond.EQ.0)) THEN
        H0=P0-P2
        H1=P1-P2
        C=Y2
        E0=Y0-C
        E1=Y1-C
        Det=H0*H1*(H0-H1)
        A=(E0*H1-H0*E1)/Det
        B=(H0*H0*E1-H1*H1*E0)/Det
        IF ((B*B).GT.(4*A*C)) THEN
          Disc=SQRT(B*B-4*A*C)
        ELSE
          DISC=0
        ENDIF
        IF (B.LT.0) Disc=-Disc
        Z=-2*C/(B+Disc)
        P3=P2+Z
        IF (ABS(P3-P1).LT.ABS(P3-P0)) THEN
          U=P1;  P1=P0;  P0=U
          V=Y1;  Y1=Y0;  Y0=V
        ENDIF
        IF (ABS(P3-P2).LT.ABS(P3-P1)) THEN
          U=P2;  P2=P1;  P1=U
          V=Y2;  Y2=Y1;  Y1=V
        ENDIF
        P2=P3
        Y2=F(P2)
        RelErr=ABS(Z)/(ABS(P2)+Small)
        IF (RelErr.LT.Delta).AND.(ABS(Y2).LT.Epsilon) Cond=1
        K=K+1
        WRITE(9,1000) K,P2,Y2
        GOTO 10
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(I2,4X,F15.7,4X,F15.7)
      END

      SUBROUTINE INPUTS(P0,P1,P2)
      INTEGER I
      REAL P0,P1,P2
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'MULLER`S METHOD IS USED TO FIND A ZERO OF THE FUNCTION'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'THREE INITIAL APPROXIMATIONS P0, P1 AND P2 ARE NEEDED.'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER P0 = '
      READ(9,*) P0
      WRITE(9,*)'ENTER P1 = '
      READ(9,*) P1
      WRITE(9,*)'ENTER P2 = '
      READ(9,*) P2
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(P0,P1,P2,P3,Z,K,Cond)
      INTEGER Cond,I,K
      REAL P0,P1,P2,P3,Y3,Z
      Y3=F(P3)
      DO 10 I=1,15
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'MULLER`S METHOD WAS USED TO FIND A ZERO OF THE FUNCTION
     +'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'THE THREE INITIAL APPROXIMATIONS WERE:'
      WRITE(9,*)' '
      WRITE(9,*)'P0 =',P0,'    P1 =',P1,'    P2 =',P2
      WRITE(9,*)' '
      WRITE(9,*)'AFTER ',K,' ITERATIONS AN APPROXIMATE VALUE FOR A ZERO
     +IS:'
      WRITE(9,*)' '
      WRITE(9,*)' P =',P3
      WRITE(9,*)' '
      WRITE(9,*)'  F(',P3,'  ) = ',Y3
      WRITE(9,*)' '
      WRITE(9,*)'    ',ABS(Z),'   IS THE ESTIMATED ACCURACY'
      WRITE(9,*)' '
      IF (Y3.EQ.0) THEN
        WRITE(9,*)'THE COMPUTED FUNCTION VALUE IS EXACTLY ZERO!'
        WRITE(9,*)' '
      ENDIF
      IF (Cond.EQ.1) THEN
        WRITE(9,*)'THE ZERO WAS FOUND AND IS'
        WRITE(9,*)'WITHIN THE DESIRED TOLERANCES.'
      ELSE
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL BECAUSE THE'
        WRITE(9,*)'MAXIMUM NUMBER OF ITERATIONS WAS EXCEEDED.'
      ENDIF
      RETURN
      END
