      PROGRAM LUFACTOR
C     ----------------------------------------------------------------
C     Alg3'3.for   FORTRAN program for implementing Algorithm 3.3
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 3.3 (PA = LU Factorization with Pivoting).
C     Section   3.6, Triangular Factorizaton, Page 175
C     ----------------------------------------------------------------
      PARAMETER(MaxR=10)
      INTEGER InRC,N,Row
      REAL A,A1,B,DET,X
      CHARACTER Ans*1,Ach*1,Bch*1,Xch*1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR)
      DIMENSION B(1:MaxR),Row(1:MaxR),X(1:MaxR)
10    CALL INPUTS(A,A1,B,N,InRC)
      CALL FACTOR(A,Row,N,DET)
20    WRITE(9,*)' '
      WRITE(9,*)'ENTER THE COLUMN VECTOR  B:'
      CALL VECIN('B',B,N)
      IF (DET.NE.0) THEN
        CALL SOLVE(A,Row,B,X,N)
      ENDIF
        CALL RESULTS(A1,B,X,N,DET)
      IF (DET.NE.0) THEN
        WRITE(9,*)'WANT TO SOLVE A*X=B WITH A NEW VECTOR B ? <Y/N> '
        READ(9,'(A)') Ans
        IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') GOTO 20
      ENDIF
      WRITE(9,*)'WANT TO SOLVE ANOTHER LINEAR SYSTEM ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') GOTO 10
      END

      SUBROUTINE FACTOR(A,Row,N,DET)
      PARAMETER(MaxR=10)
      INTEGER C,J,InRC,K,N,P,Row,RowK,RowP,T
      REAL A,A1,B,DET,X
      CHARACTER Ans*1,Ach*1,Bch*1,Xch*1
      DIMENSION A(1:MaxR,1:MaxR),Row(1:MaxR)
      DET=1
      DO J=1,N
        Row(J)=J
      ENDDO
      DO P=1,(N-1)
        DO K=(P+1),N
          IF (ABS(A(Row(K),P)).GT.ABS(A(Row(P),P))) THEN
            T=Row(P)
            Row(P)=Row(K)
            Row(K)=T
            DET=-DET
          ENDIF
        ENDDO
        DET=DET*A(Row(P),P)
        IF (DET.EQ.0) RETURN
        DO K=(P+1),N
          RowK=Row(K)
          RowP=Row(P)
          A(RowK,P)=A(RowK,P)/A(RowP,P)
          DO C=(P+1),N
            A(RowK,C)=A(RowK,C)-A(RowK,P)*A(RowP,C)
          ENDDO
        ENDDO
      ENDDO
      DET=DET*A(Row(N),N)
      RETURN
      END

      SUBROUTINE SOLVE(A,Row,B,X,N)
      PARAMETER(MaxR=10)
      INTEGER C,K,N,Row,RowK
      REAL A,B,Sum,X
      DIMENSION A(1:MaxR,1:MaxR),B(1:MaxR),Row(1:MaxR),X(1:MaxR)
      DO K=1,N
        IF (A(Row(K),K).EQ.0) RETURN
      ENDDO
      X(1)=B(Row(1))
      DO K=2,N
        Sum=0
        RowK=Row(K)
        DO C=1,(K-1)
          Sum=Sum+A(RowK,C)*X(C)
        ENDDO
        X(K)=B(RowK)-Sum
      ENDDO
      X(N)=X(N)/A(Row(N),N)
      DO K=(N-1),1,-1
        Sum=0
        RowK=Row(K)
        DO C=(K+1),N
          Sum=Sum+A(RowK,C)*X(C)
        ENDDO
        X(K)=(X(K)-Sum)/A(RowK,K)
      ENDDO
      RETURN
      END

      SUBROUTINE XACTOR(A,Row,N,DET)
      PARAMETER(MaxR=10)
      INTEGER C,J,InRC,K,N,P,Row,RowK,RowP,T
      REAL A,A1,B,DET,X
      CHARACTER Ans*1,Ach*1,Bch*1,Xch*1
      DIMENSION A(1:MaxR,1:MaxR),Row(1:MaxR)
      DET=1
      DO 10 J=1,N
        Row(J)=J
10    CONTINUE
      DO 50 P=1,(N-1)
        DO 20 K=(P+1),N
        IF (ABS(A(Row(K),P)).GT.ABS(A(Row(P),P))) THEN
          T=Row(P)
          Row(P)=Row(K)
          Row(K)=T
          DET=-DET
        ENDIF
20      CONTINUE
        DET=DET*A(Row(P),P)
        IF (DET.EQ.0) RETURN
        DO 40 K=(P+1),N
          RowK=Row(K)
          RowP=Row(P)
          A(RowK,P)=A(RowK,P)/A(RowP,P)
          DO 30 C=(P+1),N
            A(RowK,C)=A(RowK,C)-A(RowK,P)*A(RowP,C)
30        CONTINUE
40      CONTINUE
50    CONTINUE
      DET=DET*A(Row(N),N)
      RETURN
      END

      SUBROUTINE XOLVE(A,Row,B,X,N)
      PARAMETER(MaxR=10)
      INTEGER C,K,N,Row,RowK
      REAL A,B,Sum,X
      DIMENSION A(1:MaxR,1:MaxR),B(1:MaxR),Row(1:MaxR),X(1:MaxR)
      DO 10 K=1,N
        IF (A(Row(K),K).EQ.0) RETURN
10    CONTINUE
      X(1)=B(Row(1))
      DO 30 K=2,N
        Sum=0
        RowK=Row(K)
        DO 20 C=1,(K-1)
          Sum=Sum+A(RowK,C)*X(C)
20      CONTINUE
        X(K)=B(RowK)-Sum
30    CONTINUE
      X(N)=X(N)/A(Row(N),N)
      DO 50 K=(N-1),1,-1
        Sum=0
        RowK=Row(K)
        DO 40 C=(K+1),N
          Sum=Sum+A(RowK,C)*X(C)
40      CONTINUE
        X(K)=(X(K)-Sum)/A(RowK,K)
50    CONTINUE
      RETURN
      END

      SUBROUTINE INPUTS(A,A1,B,N,InRC)
      PARAMETER(MaxR=10)
      INTEGER Col,I,InRC,N,Row
      REAL A,B
      CHARACTER Ans*1,Ach*1
      DIMENSION A(1:MaxR,1:MaxR),A1(1:MaxR,1:MaxR),B(1:MaxR)
      DO 10 I=1,12
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'SOLUTION OF A LINEAR SYSTEM:'
      WRITE(9,*)' '
      WRITE(9,*)'      A*X = B'
      WRITE(9,*)' '
      WRITE(9,*)'THE TRIANGULAR FACTORIZATION L*U = P*A IS CONSTRUCTED'
      WRITE(9,*)' '
      WRITE(9,*)'FIRST,  THE SOLUTION  Y  TO  L*Y = P*B  IS FOUND,'
      WRITE(9,*)' '
      WRITE(9,*)'SECOND, THE SOLUTION  X  TO  U*X = Y    IS FOUND.'
      WRITE(9,*)' '
      WRITE(9,*)'A  IS AN  N BY N  NONSINGULAR MATRIX.'
      WRITE(9,*)' '
      WRITE(9,*)'B  IS AN  N  DIMENSIONAL VECTOR OF CONSTANTS.'
      WRITE(9,*)' '
      WRITE(9,*)'X  IS THE N DIMENSIONAL SOLUTION VECTOR.'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE NUMBER OF EQUATIONS N = '
      READ(9,*) N
      InRC=0
      WRITE(9,*)'WANT TO INPUT COLUMNS ? <Y/N> '
      READ(9,'(A)') Ans
      IF (Ans.EQ.'Y' .OR. Ans.EQ.'y') THEN
        InRC=1
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE COEFFICIENT MATRIX  A:'
      CALL MATIN('A',A,N,InRC)
      DO 30 Row=1,N
      DO 20 Col=1,N
        A1(Row,Col)=A(Row,Col)
20    CONTINUE
30    CONTINUE
      RETURN
      END

      SUBROUTINE MATIN(Ach,A,N,InRC)
      PARAMETER(MaxR=10)
      INTEGER Col,InRC,N,Row
      REAL A
      CHARACTER Ach*1
      DIMENSION A(1:MaxR,1:MaxR)
      DO 20 Row=1,N
      DO 10 Col=1,N
        A(Row,Col)=0
10    CONTINUE
20    CONTINUE
      WRITE(9,*)'GIVE THE ELEMENTS OF THE MATRIX ',Ach
      WRITE(9,*)' '
      IF (InRC.EQ.0) THEN
        DO 40 Row=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF ROW ',Row
          WRITE(9,*)' '
        DO 30 Col=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
30      CONTINUE
40      CONTINUE
      ELSE
        DO 60 Col=1,N
          WRITE(9,*)'GIVE THE ELEMENTS OF COLUMN ',Col
          WRITE(9,*)' '
        DO 50 Row=1,N
          WRITE(9,*) Ach,'(',Row,',',Col,') = '
          READ(9,*) A(Row,Col)
50      CONTINUE
60      CONTINUE
      ENDIF
      RETURN
      END

      SUBROUTINE VECIN(Bch,B,N)
      PARAMETER(MaxR=10)
      INTEGER N,Row
      REAL A
      CHARACTER Bch*1
      DIMENSION B(1:MaxR)
      WRITE(9,*)'GIVE THE ELEMENTS OF THE COLUMN VECTOR ',Bch
      WRITE(9,*)' '
      DO 10 Row=1,N
        WRITE(9,*) Bch,'(',Row,') = '
        READ(9,*) B(Row)
10    CONTINUE
      RETURN
      END

      SUBROUTINE MATPRINT(A,N)
      PARAMETER(MaxR=10)
      INTEGER Col,N,Row
      REAL A
      DIMENSION A(1:MaxR,1:MaxR)
      DO 10 Row=1,N
        WRITE(9,999) ( A(Row,Col), Col=1,N )
10    CONTINUE
999   FORMAT(1X,5F15.7,4X)
      RETURN
      END

      SUBROUTINE VECPRINT(Bch,Xch,B,X,N)
      PARAMETER(MaxR=10)
      INTEGER N,Row
      REAL B,X
      CHARACTER Bch*1,Xch*1
      DIMENSION B(1:MaxR),X(1:MaxR)
      WRITE(9,*) Bch,' COEFFICIENT VECTOR                ',Xch,
     +' SOLUTION VECTOR'
      WRITE(9,*)
      DO 10 Row=1,N
        WRITE(9,*) Bch,'(',Row,') =',B(Row),'                     ',
     +Xch,'(',Row,') =',X(Row)
10    CONTINUE
      RETURN
      END

      SUBROUTINE RESULTS(A,B,X,N,DET)
      PARAMETER(MaxR=10)
      INTEGER I,N
      REAL A,B,DET,X
      CHARACTER Bch*1,Xch*1
      DIMENSION A(1:MaxR,1:MaxR),B(1:MaxR),X(1:MaxR)
      DO 10 I=1,15
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'COMPUTATION OF THE SOLUTION FOR THE LINEAR SYSTEM A*X =
     + B.'
      IF (DET.NE.0) THEN
      WRITE(9,*)'THE TRIANGULAR FACTORIZATION L*U = P*A WAS CONSTRUCTED'
      WRITE(9,*)'FIRST,  THE SOLUTION  Y  TO  L*Y = P*B WAS FOUND,'
      WRITE(9,*)'SECOND, THE SOLUTION  X  TO  U*X = Y   WAS FOUND.'
      ENDIF
      WRITE(9,*)
      CALL MATPRINT(A,N)
      IF (DET.EQ.0) THEN
        WRITE(9,*)
        WRITE(9,*)'THE MATRIX IS SINGULAR.'
        WRITE(9,*)
        WRITE(9,*)'A ZERO PIVOT ELEMENT WAS ENCOUNTERED.'
        WRITE(9,*)' '
        WRITE(9,*)'THE MATRIX DOES NOT HAVE TRIANGULAR FACTORIZATION.'
        WRITE(9,*)' '
        WRITE(9,*)'THE METHOD DOES NOT APPLY.'
      ELSE
        WRITE(9,*)
        CALL VECPRINT('B','X',A,X,N)
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'THE DETERMINANT`S VALUE IS  DET A = ',DET
      RETURN
      END
