      PROGRAM NEPOLY
C     ----------------------------------------------------------------
C     Alg4'5.for   FORTRAN program for implementing Algorithm 4.5
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 4.5 (Newton Interpolation Polynomial).
C     Section   4.4, Newton Polynomials, Page 234
C     ----------------------------------------------------------------
      PARAMETER(MaxN=15)
      INTEGER N
      REAL A,P,T,X,Y
      CHARACTER ANS*1
      DIMENSION A(0:MaxN),X(0:MaxN),Y(0:MaxN)
      EXTERNAL P
10    CALL GETDATAS(X,Y,N)
      CALL DIVDIFFS(X,Y,A,N)
20    CALL EVALUATE(X,Y,A,N,T)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO EVALUATE P(T) AGAIN ? '
      READ(9,'(A)') ANS
      IF(ANS .EQ. 'Y' .OR. ANS .EQ. 'y') GOTO 20
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO EVALUATE ANOTHER POLYNOMIAL ? '
      READ(9,'(A)') ANS
      IF(ANS .EQ. 'Y' .OR. ANS .EQ. 'y') GOTO 10
      STOP
      END

      SUBROUTINE DIVDIFFS(X,Y,A,N)
      PARAMETER(MaxN=15)
      INTEGER J,K,N
      REAL A,D,X,Y
      DIMENSION A(0:MaxN),D(0:MaxN,0:MaxN),X(0:MaxN),Y(0:MaxN)
      DO K=0,N
        D(K,0)=Y(K)
      ENDDO
      DO J=1,N
        DO K=J,N
          D(K,J)=(D(K,J-1)-D(K-1,J-1))/(X(K)-X(K-J))
        ENDDO
      ENDDO
      DO K=0,N
        A(K)=D(K,K)
      ENDDO
      RETURN
      END

      REAL FUNCTION P(A,X,N,T)
      PARAMETER(MaxN=15)
      INTEGER K,N
      REAL A,Sum,T,X
      DIMENSION A(0:MaxN),X(0:MaxN)
        Sum=A(N)
        DO K=(N-1),0,-1
          Sum=Sum*(T-X(K))+A(K)
        ENDDO
        P=Sum
        RETURN
      END

      SUBROUTINE XDIVDIFFS(X,Y,A,N)
      PARAMETER(MaxN=15)
      INTEGER J,K,N
      REAL A,D,X,Y
      DIMENSION A(0:MaxN),D(0:MaxN,0:MaxN),X(0:MaxN),Y(0:MaxN)
      DO 10 K=0,N
        D(K,0)=Y(K)
10    CONTINUE
      DO 30 J=1,N
        DO 20 K=J,N
          D(K,J)=(D(K,J-1)-D(K-1,J-1))/(X(K)-X(K-J))
20      CONTINUE
30    CONTINUE
      DO 40 K=0,N
        A(K)=D(K,K)
40    CONTINUE
      RETURN
      END

      REAL FUNCTION XP(A,X,N,T)
      PARAMETER(MaxN=15)
      INTEGER K,N
      REAL A,Sum,T,X
      DIMENSION A(0:MaxN),X(0:MaxN)
      Sum=A(N)
      DO 10 K=(N-1),0,-1
        Sum=Sum*(T-X(K))+A(K)
10    CONTINUE
      P=Sum
      RETURN
      END

      SUBROUTINE GETDATAS(X,Y,N)
      PARAMETER(MaxN=15)
      INTEGER I,K,N
      REAL X,Y
      DIMENSION X(0:MaxN),Y(0:MaxN)
      DO 10 I=1,18
      WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'THE NEWTON POLYNOMIAL IS CONSTRUCTED:'
      WRITE(9,*)' '
      WRITE(9,*)'P(T) = A  + A [T-X ] + A [T-X ][T-X ] + A [T-X ][T-X ][
     +T-X ]'
      WRITE(9,*)'        0    1    0     2    0     1     3    0     1
     +   2'
      WRITE(9,*)'       +...+ A [T-X ][T-X ]...[T-X   ]'
      WRITE(9,*)'              N    0     1        N-1'
      WRITE(9,*)' '
      WRITE(9,*)'BASED ON THE N+1 POINTS (X ,Y ),(X ,Y ),...,(X ,Y )'
      WRITE(9,*)'                          0  0    1  1        N  N'
      WRITE(9,*)' '
      WRITE(9,*)'THE CENTERS ARE THE ABSCISSAS  X  , X  ,..., X    .'
      WRITE(9,*)'                                0    1        N-1 '
      WRITE(9,*)' '
      WRITE(9,*)'THE COEFFICIENTS ARE A , A ,..., A   , A  , AND THEY'
      WRITE(9,*)'                      0   1       N-1   N '
      WRITE(9,*)' '
      WRITE(9,*)'ARE FOUND BY CONSTRUCTING A DIVIDED DIFFERENCE TABLE.'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE DEGREE  N = '
      READ(9,*) N
      WRITE(9,*)' '
      WRITE(9,*)'NOW ENTER THE ',N+1,' POINTS'
      WRITE(9,*)' '
      DO 20 K=0,N
        WRITE(9,*)'X(',K,') = '
        READ(9,*) X(K)
        WRITE(9,*)'Y(',K,') = '
        READ(9,*) Y(K)
        WRITE(9,*)' '
20    CONTINUE
      RETURN
      END

      SUBROUTINE PRINTPOL(A,X,Y,N)
      PARAMETER(MaxN=15)
      INTEGER K,N
      REAL A,X,Y
      DIMENSION A(0:MaxN),X(0:MaxN),Y(0:MaxN)
        WRITE(9,*)' '
      IF (N.EQ.1) THEN
        WRITE(9,*)'P(T) = A  + A [T-X ]'
        WRITE(9,*)'        0    1    0 '
      ELSEIF (N.EQ.2) THEN
        WRITE(9,*)'P(T) = A  + A [T-X ] + A [T-X ][T-X ]'
        WRITE(9,*)'        0    1    0     2    0     1 '
      ELSEIF (N.EQ.3) THEN
        WRITE(9,*)'P(T) = A  + A [T-X ] + A [T-X ][T-X ] + A [T-X ][T-X
     +][T-X ]'
        WRITE(9,*)'        0    1    0     2    0     1     3    0     1
     +     2 '
      ELSE
        WRITE(9,*)'P(T) = A  + A [T-X ] + A [T-X ][T-X ] + A [T-X ][T-X
     +][T-X ]'
        WRITE(9,*)'        0    1    0     2    0     1     3    0     1
     +     2'
        WRITE(9,*)'       +...+ A [T-X ][T-X ] ... [T-X ]'
        WRITE(9,*)'    ',N,'    0     1',(N-1)
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'            THE COEFFICIENTS:   THE ABSCISSAS:
     +      THE ORDINATES:'
      WRITE(9,*)'  K             A(K)                X(K)      
     +          Y(K)'
      WRITE(9,*)' '
      DO 10 K=0,N
        WRITE(9,999) K,A(K),X(K),Y(K)
10    CONTINUE
999   FORMAT(1X,I2,3F20.7)
      RETURN
      END

      SUBROUTINE EVALUATE(X,Y,A,N,T)
      PARAMETER(MaxN=15)
      INTEGER N
      REAL A,P,T,X
      DIMENSION A(0:MaxN),X(0:MaxN),Y(0:MaxN)
      EXTERNAL P
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      CALL PRINTPOL(A,X,Y,N)
      WRITE(9,*)' '
      WRITE(9,*)'NOW EVALUATE  P(T)'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER A VALUE   T = '
      READ(9,*) T
      WRITE(9,*)' '
      WRITE(9,*)'THE VALUE OF THE NEWTON POLYNOMIAL IS '
      WRITE(9,*)' '
      Poly=P(A,X,N,T)
      WRITE(9,*)'P(',T,'  )  = ',Poly
      RETURN
      END
