      PROGRAM CURVEFIT
C     ----------------------------------------------------------------
C     Alg5'3.for   FORTRAN program for implementing Algorithm 5.3
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 5.3 (Non-linear Curve Fitting).
C     Section   5.2, Curve Fitting, Page 280
C     ----------------------------------------------------------------
      PARAMETER(MaxN=100)
      INTEGER Ctype,N
      REAL A,B,C,D,F,L,X,Y,X1,Y1
      CHARACTER ANS*1
      DIMENSION X(1:MaxN),X1(1:MaxN),Y(1:MaxN),Y1(1:MaxN)
      EXTERNAL F
10    CALL GETDATA(X,Y,X1,Y1,N)
20    CALL CURTYPE(Y1,N,Ctype,L)
      CALL CHANGEV(X1,Y1,X,Y,N,Ctype,L)
      CALL REGRESS(X,Y,N,A,B)
      CALL CONSTAN(A,B,C,D,Ctype)
      CALL RESULTS(X1,Y1,N,A,B,C,D,Ctype,L)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO FIT ANOTHER CURVE TO THIS DATA ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 20
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO FIT A CURVE FOR SOME NEW DATA ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X,A,B,C,D,Ctype,L)
      INTEGER Ctype
      REAL A,B,C,D,L,X
        IF (Ctype.EQ.1) THEN
          F=A*X+B
        ELSEIF (Ctype.EQ.2) THEN
          F=A/X+B
        ELSEIF (Ctype.EQ.3) THEN
          F=D/(X+C)
        ELSEIF (Ctype.EQ.4) THEN
          F=1/(A*X+B)
        ELSEIF (Ctype.EQ.5) THEN
          F=X/(A+B*X)
        ELSEIF (Ctype.EQ.6) THEN
          F=A*ALOG(X)+B
        ELSEIF (Ctype.EQ.7) THEN
          F=C*EXP(A*X)
        ELSEIF (Ctype.EQ.8) THEN
          F=C*EXP(A*ALOG(X))
        ELSEIF (Ctype.EQ.9) THEN
          F=1/((A*X+B)*(A*X+B))
        ELSEIF (Ctype.EQ.10) THEN
          F=C*X*EXP(-D*X)
        ELSEIF (Ctype.EQ.11) THEN
          F=L/(1.0+C*EXP(A*X))
        ENDIF
        RETURN
      END

      SUBROUTINE CHANGEV(X1,Y1,X,Y,N,Ctype,L)
      PARAMETER(MaxN=100)
      INTEGER Ctype,K,N
      REAL L,X,Y,X1,Y1
      DIMENSION X(1:MaxN),X1(1:MaxN),Y(1:MaxN),Y1(1:MaxN)
      IF (Ctype.EQ.1) THEN
        DO K=1,N
          X(K)=X1(K)
          Y(K)=Y1(K)
        ENDDO
      ELSEIF (Ctype.EQ.2) THEN
        DO K=1,N
          X(K)=1.0/X1(K)
          Y(K)=Y1(K)
        ENDDO
      ELSEIF (Ctype.EQ.3) THEN
        DO K=1,N
          X(K)=X1(K)*Y1(K)
          Y(K)=Y1(K)
        ENDDO
      ELSEIF (Ctype.EQ.4) THEN
        DO K=1,N
          X(K)=X1(K)
          Y(K)=1.0/Y1(K)
        ENDDO
      ELSEIF (Ctype.EQ.5) THEN
        DO K=1,N
          X(K)=1.0/X1(K)
          Y(K)=1.0/Y1(K)
        ENDDO
      ELSEIF (Ctype.EQ.6) THEN
        DO K=1,N
          X(K)=ALOG(X1(K))
          Y(K)=Y1(K)
        ENDDO
      ELSEIF (Ctype.EQ.7) THEN
        DO K=1,N
          X(K)=X1(K)
          Y(K)=ALOG(Y1(K))
        ENDDO
      ELSEIF (Ctype.EQ.8) THEN
        DO K=1,N
          X(K)=ALOG(X1(K))
          Y(K)=ALOG(Y1(K))
        ENDDO
      ELSEIF (Ctype.EQ.9) THEN
        DO K=1,N
          X(K)=X1(K)
          Y(K)=1.0/SQRT(Y1(K))
        ENDDO
      ELSEIF (Ctype.EQ.10) THEN
        DO K=1,N
          X(K)=X1(K)
          Y(K)=ALOG(Y1(K)/X1(K))
        ENDDO
      ELSEIF (Ctype.EQ.11) THEN
        DO K=1,N
          X(K)=X1(K)
          Y(K)=ALOG(L/Y1(K)-1.0)
        ENDDO
      ENDIF
      RETURN
      END

      SUBROUTINE REGRESS(X,Y,N,A,B)
      PARAMETER(MaxN=100)
      INTEGER K,N
      REAL A,B,SumX,SumXY,X,Xmean,Y,Ymean
      DIMENSION X(1:MaxN),Y(1:MaxN)
      Xmean=0.0
      DO K=1,N
        Xmean=Xmean+X(K)
      ENDDO
      Xmean=Xmean/N
      Ymean=0.0
      DO K=1,N
        Ymean=Ymean+Y(K)
      ENDDO
      Ymean=Ymean/N
      SumX=0.0
      DO K=1,N
        SumX=SumX+(X(K)-Xmean)*(X(K)-Xmean)
      ENDDO
      SumXY=0.0
      DO K=1,N
        SumXY=SumXY+(X(K)-Xmean)*(Y(K)-Ymean)
      ENDDO
      A=SumXY/SumX
      B=Ymean-A*Xmean
      RETURN
      END

      SUBROUTINE CONSTAN(A,B,C,D,Ctype)
      INTEGER Ctype
      REAL A,B,C,D
      IF (Ctype.EQ.3) THEN
        C=-1.0/A
        D=-B/A
      ELSEIF (Ctype.EQ.7) THEN
        C=EXP(B)
      ELSEIF (Ctype.EQ.8) THEN
        C=EXP(B)
      ELSEIF (Ctype.EQ.10) THEN
        C=EXP(B)
        D=-A
      ELSEIF (Ctype.EQ.11) THEN
        C=EXP(B)
      ENDIF
      RETURN
      END

      REAL FUNCTION XF(X,A,B,C,D,Ctype,L)
      INTEGER Ctype
      REAL A,B,C,D,L,X
      IF (Ctype.EQ.1) THEN
        F=A*X+B
      ELSEIF (Ctype.EQ.2) THEN
        F=A/X+B
      ELSEIF (Ctype.EQ.3) THEN
        F=D/(X+C)
      ELSEIF (Ctype.EQ.4) THEN
        F=1/(A*X+B)
      ELSEIF (Ctype.EQ.5) THEN
        F=X/(A+B*X)
      ELSEIF (Ctype.EQ.6) THEN
        F=A*ALOG(X)+B
      ELSEIF (Ctype.EQ.7) THEN
        F=C*EXP(A*X)
      ELSEIF (Ctype.EQ.8) THEN
        F=C*EXP(A*ALOG(X))
      ELSEIF (Ctype.EQ.9) THEN
        F=1/((A*X+B)*(A*X+B))
      ELSEIF (Ctype.EQ.10) THEN
        F=C*X*EXP(-D*X)
      ELSEIF (Ctype.EQ.11) THEN
        F=L/(1.0+C*EXP(A*X))
      ENDIF
      RETURN
      END

      SUBROUTINE XHANGEV(X1,Y1,X,Y,N,Ctype,L)
      PARAMETER(MaxN=100)
      INTEGER Ctype,K,N
      REAL L,X,Y,X1,Y1
      DIMENSION X(1:MaxN),X1(1:MaxN),Y(1:MaxN),Y1(1:MaxN)
      IF (Ctype.EQ.1) THEN
        DO 10 K=1,N
          X(K)=X1(K)
          Y(K)=Y1(K)
10      CONTINUE
      ELSEIF (Ctype.EQ.2) THEN
        DO 20 K=1,N
          X(K)=1.0/X1(K)
          Y(K)=Y1(K)
20      CONTINUE
      ELSEIF (Ctype.EQ.3) THEN
        DO 30 K=1,N
          X(K)=X1(K)*Y1(K)
          Y(K)=Y1(K)
30      CONTINUE
      ELSEIF (Ctype.EQ.4) THEN
        DO 40 K=1,N
          X(K)=X1(K)
          Y(K)=1.0/Y1(K)
40      CONTINUE
      ELSEIF (Ctype.EQ.5) THEN
        DO 50 K=1,N
          X(K)=1.0/X1(K)
          Y(K)=1.0/Y1(K)
50      CONTINUE
      ELSEIF (Ctype.EQ.6) THEN
        DO 60 K=1,N
          X(K)=ALOG(X1(K))
          Y(K)=Y1(K)
60      CONTINUE
      ELSEIF (Ctype.EQ.7) THEN
        DO 70 K=1,N
          X(K)=X1(K)
          Y(K)=ALOG(Y1(K))
70      CONTINUE
      ELSEIF (Ctype.EQ.8) THEN
        DO 80 K=1,N
          X(K)=ALOG(X1(K))
          Y(K)=ALOG(Y1(K))
80      CONTINUE
      ELSEIF (Ctype.EQ.9) THEN
        DO 90 K=1,N
          X(K)=X1(K)
          Y(K)=1.0/SQRT(Y1(K))
90      CONTINUE
      ELSEIF (Ctype.EQ.10) THEN
        DO 100 K=1,N
          X(K)=X1(K)
          Y(K)=ALOG(Y1(K)/X1(K))
100     CONTINUE
      ELSEIF (Ctype.EQ.11) THEN
        DO 110 K=1,N
          X(K)=X1(K)
          Y(K)=ALOG(L/Y1(K)-1.0)
110     CONTINUE
      ENDIF
      RETURN
      END

      SUBROUTINE XEGRESS(X,Y,N,A,B)
      PARAMETER(MaxN=100)
      INTEGER K,N
      REAL A,B,SumX,SumXY,X,Xmean,Y,Ymean
      DIMENSION X(1:MaxN),Y(1:MaxN)
      Xmean=0.0
      DO 10 K=1,N
        Xmean=Xmean+X(K)
10    CONTINUE
      Xmean=Xmean/N
      Ymean=0.0
      DO 20 K=1,N
        Ymean=Ymean+Y(K)
20    CONTINUE
      Ymean=Ymean/N
      SumX=0.0
      DO 30 K=1,N
        SumX=SumX+(X(K)-Xmean)*(X(K)-Xmean)
30    CONTINUE
      SumXY=0.0
      DO 40 K=1,N
        SumXY=SumXY+(X(K)-Xmean)*(Y(K)-Ymean)
40    CONTINUE
      A=SumXY/SumX
      B=Ymean-A*Xmean
      RETURN
      END

      SUBROUTINE XCONSTAN(A,B,C,D,Ctype)
      INTEGER Ctype
      REAL A,B,C,D
      IF (Ctype.EQ.3) THEN
        C=-1.0/A
        D=-B/A
      ELSEIF (Ctype.EQ.7) THEN
        C=EXP(B)
      ELSEIF (Ctype.EQ.8) THEN
        C=EXP(B)
      ELSEIF (Ctype.EQ.10) THEN
        C=EXP(B)
        D=-A
      ELSEIF (Ctype.EQ.11) THEN
        C=EXP(B)
      ENDIF
      RETURN
      END

      SUBROUTINE GETDATA(X,Y,X1,Y1,N)
      PARAMETER(MaxN=100)
      INTEGER I,K,N
      REAL X,Y,X1,Y1
      DIMENSION X(1:MaxN),X1(1:MaxN),Y(1:MaxN),Y1(1:MaxN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'    THE METHOD OF "DATA LINEARIZATION" IS USED'
      WRITE(9,*)' '
      WRITE(9,*)'TO FIT ONE OF THE CURVES    Y=A*X+B,       Y=A/X+B'
      WRITE(9,*)' '
      WRITE(9,*)'Y=D/(X+C),   Y=1/(A*X+B),   Y=X/(A+B*X),   Y=A*Ln(X)+B'
      WRITE(9,*)' '
      WRITE(9,*)'Y=C*Exp(A*X),    Y=C*X^A,   Y=(A*X+B)^-2'
      WRITE(9,*)' '
      WRITE(9,*)'Y=C*X*Exp(-D*X), Y=L/(1+C*Exp(A*X))'
      WRITE(9,*)' '
      WRITE(9,*)'TO THE N DATA POINTS (X ,Y ), (X ,Y ),..., (X ,Y ).'
      WRITE(9,*)'                       1  1     2  2         N  N  '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER NUMBER OF POINTS N = '
      READ(9,*) N
      IF (N.LT.2) THEN
        N=2
      ENDIF
      WRITE(9,*)' '
      DO 20 K=1,N
        WRITE(9,*)'X(',K,') = '
        READ(9,*) X(K)
        X1(K)=X(K)
        WRITE(9,*)'Y(',K,') = '
        READ(9,*) Y(K)
        Y1(K)=Y(K)
        WRITE(9,*)' '
20    CONTINUE
      RETURN
      END

      SUBROUTINE CURTYPE(Y,N,Ctype,L)
      PARAMETER(MaxN=100)
      INTEGER Ctype,K,N
      REAL L,LM,Y
      DIMENSION Y(1:MaxN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'      WHICH CURVE DO YOU WANT TO FIT ?'
      WRITE(9,*)' '
      WRITE(9,*)' 1:   Y = A*X + B'
      WRITE(9,*)' '
      WRITE(9,*)' 2:   Y = A/X + B'
      WRITE(9,*)' '
      WRITE(9,*)' 3:   Y = D/(X + C)'
      WRITE(9,*)' '
      WRITE(9,*)' 4:   Y = 1/(A*X + B)'
      WRITE(9,*)' '
      WRITE(9,*)' 5:   Y = X/(A + B*X)'
      WRITE(9,*)' '
      WRITE(9,*)' 6:   Y = A*Ln(X) + B'
      WRITE(9,*)' '
      WRITE(9,*)' 7:   Y = C*Exp(A*X)'
      WRITE(9,*)' '
      WRITE(9,*)' 8:   Y = C*X^A'
      WRITE(9,*)' '
      WRITE(9,*)' 9:   Y = (A*X + B)^-2'
      WRITE(9,*)' '
      WRITE(9,*)'10:   Y = C*X*EXP(-D*X)'
      WRITE(9,*)' '
      WRITE(9,*)'11:   Y = L/(1+C*EXP(A*X))'
      WRITE(9,*)' '
      WRITE(9,*)'      CHOOSE TYPE OF CURVE <1-11> '
      READ(9,*) Ctype
      IF (Ctype.EQ.11) THEN
        DO 20 I=1,20
          WRITE(9,*)' '
20      CONTINUE
        WRITE(9,*)' '
        WRITE(9,*)'     YOU CHOSE THE CURVE  Y = L/(1+C*EXP(A*X))'
        WRITE(9,*)' '
        WRITE(9,*)'THE LIMITING  VALUE  L =  LIM   Y(X)  MUST BE GIVEN.'
        WRITE(9,*)'                         X ->INF'
        WRITE(9,*)' '
        WRITE(9,*)'ENTER  THE  VALUE    L = '
        READ(9,*) L
        LM=ABS(Y(1))
        DO 30 K=2,N
          IF (LM.LT.ABS(Y(K))) THEN
            LM=ABS(Y(K))
          ENDIF
30      CONTINUE
        IF (L.LE.LM) THEN
          L=LM*1.0001
        ENDIF
      ENDIF
      RETURN
      END

      SUBROUTINE RESULTS(X,Y,N,A,B,C,D,Ctype,L)
      PARAMETER(MaxN=100)
      INTEGER Ctype,I,K,N
      REAL A,B,C,D,Err,F,L,X,Y,Z
      DIMENSION X(1:MaxN),Y(1:MaxN)
      EXTERNAL F
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      IF (Ctype.EQ.1) THEN
        WRITE(9,*)'  F(X) = A*X +B'
        WRITE(9,*)' '
        WRITE(9,*)'    A  = ',A
        WRITE(9,*)' '
        WRITE(9,*)'    B  = ',B
      ELSEIF (Ctype.EQ.2) THEN
        WRITE(9,*)'  F(X) = A/X + B'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    B  =',B
      ELSEIF (Ctype.EQ.3) THEN
        WRITE(9,*)'  F(X) = D/(X + C)'
        WRITE(9,*)' '
        WRITE(9,*)'    C  =',C
        WRITE(9,*)' '
        WRITE(9,*)'    D  =',D
      ELSEIF (Ctype.EQ.4) THEN
        WRITE(9,*)'  F(X) = 1/(A*X + B)'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    B  =',B
      ELSEIF (Ctype.EQ.5) THEN
        WRITE(9,*)'  F(X) = X/(A + B*X)'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    B  =',B
      ELSEIF (Ctype.EQ.6) THEN
        WRITE(9,*)'  F(X) = A*Ln(X) + B'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    B  =',B
      ELSEIF (Ctype.EQ.7) THEN
        WRITE(9,*)'  F(X) = C*Exp(A*X)'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    C  =',C
      ELSEIF (Ctype.EQ.8) THEN
        WRITE(9,*)'  F(X) = C*X^A'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    C  =',C
      ELSEIF (Ctype.EQ.9) THEN
        WRITE(9,*)'  F(X) = (A*X + B)^-2'
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
        WRITE(9,*)' '
        WRITE(9,*)'    B  =',B
      ELSEIF (Ctype.EQ.10) THEN
        WRITE(9,*)'  F(X) = C*X*EXP(-D*X)'
        WRITE(9,*)' '
        WRITE(9,*)'    C  =',C
        WRITE(9,*)' '
        WRITE(9,*)'    D  =',D
      ELSEIF (Ctype.EQ.11) THEN
        WRITE(9,*)'  F(X) = L/(1+C*EXP(A*X))'
        WRITE(9,*)' '
        WRITE(9,*)'    L  =',L
        WRITE(9,*)' '
        WRITE(9,*)'    C  =',C
        WRITE(9,*)' '
        WRITE(9,*)'    A  =',A
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'    K       X           Y           F(X )        Error'
      WRITE(9,*)'             K           K             K'
      WRITE(9,*)'  -----------------------------------------------------
     +----'
      DO 20 K=1,N
        Z=F(X(K),A,B,C,D,Ctype,L)
        Err=Y(K)-Z
        WRITE(9,999) K,X(K),Y(K),Z,Err
20    CONTINUE
999   FORMAT(I5,2F12.5,2F14.7)
      RETURN
      END
