      PROGRAM CUBICSPL
C     ----------------------------------------------------------------
C     Alg5'4.for   FORTRAN program for implementing Algorithm 5.4
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 5.4 (Cubic Splines).
C     Section   5.3, Interpolation by Spline Functions, Page 297
C     ----------------------------------------------------------------
      PARAMETER(MaxN=50)
      INTEGER N,Stype
      REAL A,B,C,D,DX0,DXN,DDX0,DDXN,H,M,S,T,V,X,Y
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN),D(0:MaxN),H(0:MaxN)
      DIMENSION M(0:MaxN),V(0:MaxN),X(0:MaxN),Y(0:MaxN),S(0:MaxN,0:3)
      CHARACTER ANS*1
      EXTERNAL CS
10    CALL GETDATA(X,Y,N)
20    CALL DIFFERS(X,Y,N,A,B,C,D,H,V)
      CALL SPLTYPE(Stype)
      CALL MODIFYS(A,B,C,D,H,M,V,X,N,Stype,DX0,DXN,DDX0,DDXN)
      CALL TRIDIAG(A,B,C,V,M,N)
      CALL ENDCOEF(D,H,M,N,Stype,DX0,DXN,DDX0,DDXN)
      CALL COMCOEF(D,H,M,S,Y,N)
      CALL RESULTS(S,X,Y,N,Stype,DX0,DXN,DDX0,DDXN)
30    CALL EVALUAT(CS,S,X,Y,N,Stype,DX0,DXN,DDX0,DDXN)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO EVALUATE THIS SPLINE AT ANOTHER POINT ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 30
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO FIT ANOTHER CUBIC SPLINE TO THIS DATA SET ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 20
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO FIND A CUBIC SPLINE FOR SOME NEW DATA SET ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      SUBROUTINE DIFFERS(X,Y,N,A,B,C,D,H,V)
      PARAMETER(MaxN=50)
      INTEGER K,N
      REAL A,B,C,D,H,V,X,Y
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN),D(0:MaxN)
      DIMENSION H(0:MaxN),V(0:MaxN),X(0:MaxN),Y(0:MaxN)
      H(0)= X(1)-X(0)
      D(0)=(Y(1)-Y(0))/H(0)
      DO K=1,(N-1)
        H(K)= X(K+1)-X(K)
        D(K)=(Y(K+1)-Y(K))/H(K)
        A(K)=H(K)
        B(K)=2*(H(K-1)+H(K))
        C(K)=H(K)
        V(K)=6*(D(K)-D(K-1))
      ENDDO
      RETURN
      END

      SUBROUTINE MODIFYS(A,B,C,D,H,M,V,X,N,Stype,DX0,DXN,DDX0,DDXN)
      PARAMETER(MaxN=50)
      INTEGER N,Stype
      REAL A,B,C,D,DX0,DXN,DDX0,DDXN,H,M,V
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN),D(0:MaxN)
      DIMENSION H(0:MaxN),M(0:MaxN),V(0:MaxN),X(0:MaxN)
      IF (Stype.EQ.1) THEN
        WRITE(9,*)' '
        WRITE(9,*)'ENTER THE FIRST DERIVATIVES.'
        WRITE(9,*)' '
        WRITE(9,*)'S`(',X(0),'  ) = '
        READ(9,*) DX0
        WRITE(9,*)' '
        WRITE(9,*)'S`(',X(N),'  ) = '
        READ(9,*) DXN
        WRITE(9,*)' '
        B(1)=B(1)-H(0)/2
        V(1)=V(1)-3*(D(0)-DX0)
        B(N-1)=B(N-1)-H(N-1)/2
        V(N-1)=V(N-1)-3*(DXN-D(N-1))
      ELSEIF (Stype.EQ.2) THEN
        M(0)=0
        M(N)=0
      ELSEIF (Stype.EQ.3) THEN
        B(1)=B(1)+H(0)+H(0)*H(0)/H(1)
        C(1)=C(1)-H(0)*H(0)/H(1)
        B(N-1)=B(N-1)+H(N-1)+H(N-1)*H(N-1)/H(N-2)
        A(N-2)=A(N-2)-H(N-1)*H(N-1)/H(N-2)
      ELSEIF (Stype.EQ.4) THEN
        B(1)=B(1)+H(0)
        B(N-1)=B(N-1)+H(N-1)
      ELSEIF (Stype.EQ.5) THEN
        WRITE(9,*)' '
        WRITE(9,*)'ENTER THE SECOND DERIVATIVES.'
        WRITE(9,*)' '
        WRITE(9,*)'S``(',X(0),'  ) = '
        READ(9,*) DDX0
        WRITE(9,*)' '
        WRITE(9,*)'S``(',X(N),'  ) = '
        READ(9,*) DDXN
        WRITE(9,*)' '
        V(1)=V(1)-H(0)*DDX0
        V(N-1)=V(N-1)-H(N-1)*DDXN
      ENDIF
      RETURN
      END

      SUBROUTINE TRIDIAG(A,B,C,V,M,N)
      PARAMETER(MaxN=50)
      INTEGER K,N
      REAL A,B,C,M,T,V
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN)
      DIMENSION M(0:MaxN),V(0:MaxN)
      DO K=2,(N-1)
        T=A(K-1)/B(K-1)
        B(K)=B(K)-T*C(K-1)
        V(K)=V(K)-T*V(K-1)
      ENDDO
      M(N-1)=V(N-1)/B(N-1)
      DO K=(N-2),1,-1
        M(K)=(V(K)-C(K)*M(K+1))/B(K)
      ENDDO
      RETURN
      END

      SUBROUTINE ENDCOEF(D,H,M,N,Stype,DX0,DXN,DDX0,DDXN)
      PARAMETER(MaxN=50)
      INTEGER N,Stype
      REAL D,DX0,DXN,DDX0,DDXN,H,M
      DIMENSION D(0:MaxN),H(0:MaxN),M(0:MaxN)
      IF (Stype.EQ.1) THEN
        M(0)=3*(D(0)-DX0)/H(0)-M(1)/2
        M(N)=3*(DXN-D(N-1))/H(N-1)-M(N-1)/2
      ELSEIF (Stype.EQ.2) THEN
        M(0)=0
        M(N)=0
      ELSEIF (Stype.EQ.3) THEN
        M(0)=M(1)-H(0)*(M(2)-M(1))/H(1)
        M(N)=M(N-1)+H(N-1)*(M(N-1)-M(N-2))/H(N-2)
      ELSEIF (Stype.EQ.4) THEN
        M(0)=M(1)
        M(N)=M(N-1)
      ELSEIF (Stype.EQ.5) THEN
        M(0)=DDX0
        M(N)=DDXN
      ENDIF
      RETURN
      END

      SUBROUTINE COMCOEF(D,H,M,S,Y,N)
      PARAMETER(MaxN=50)
      INTEGER I,K,N
      REAL D,H,M,S
      DIMENSION D(0:MaxN),H(0:MaxN),M(0:MaxN),Y(0:MaxN),S(0:MaxN,0:3)
      DO K=0,(N-1)
        S(K,0)=Y(K)
        S(K,1)=D(K)-H(K)*(2*M(K)+M(K+1))/6
        S(K,2)=M(K)/2
        S(K,3)=(M(K+1)-M(K))/(6*H(K))
      ENDDO
      RETURN
      END

      REAL FUNCTION CS(S,X,N,T)
      PARAMETER(MaxN=50)
      INTEGER J,K,N
      REAL S,T,W,X
      DIMENSION X(0:MaxN),S(0:MaxN,0:3)
      J=0
      K=N-1
      WHILE (J.LE.N)
        J=J+1
        IF (T.LE.X(J)) THEN
          K=J-1
          J=N+1
        ENDIF
      REPEAT
      IF (T.LE.X(0)) K=0 
      W=T-X(K)
      CS=((S(K,3)*W + S(K,2))*W + S(K,1))*W + S(K,0)
      RETURN
      END

      SUBROUTINE XIFFERS(X,Y,N,A,B,C,D,H,V)
      PARAMETER(MaxN=50)
      INTEGER K,N
      REAL A,B,C,D,H,V,X,Y
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN),D(0:MaxN)
      DIMENSION H(0:MaxN),V(0:MaxN),X(0:MaxN),Y(0:MaxN)
      H(0)= X(1)-X(0)
      D(0)=(Y(1)-Y(0))/H(0)
      DO 10 K=1,(N-1)
        H(K)= X(K+1)-X(K)
        D(K)=(Y(K+1)-Y(K))/H(K)
        A(K)=H(K)
        B(K)=2*(H(K-1)+H(K))
        C(K)=H(K)
        V(K)=6*(D(K)-D(K-1))
10    CONTINUE
      RETURN
      END

      SUBROUTINE XODIFYS(A,B,C,D,H,M,V,X,N,Stype,DX0,DXN,DDX0,DDXN)
      PARAMETER(MaxN=50)
      INTEGER N,Stype
      REAL A,B,C,D,DX0,DXN,DDX0,DDXN,H,M,V
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN),D(0:MaxN)
      DIMENSION H(0:MaxN),M(0:MaxN),V(0:MaxN),X(0:MaxN)
      IF (Stype.EQ.1) THEN
        WRITE(9,*)' '
        WRITE(9,*)'ENTER THE FIRST DERIVATIVES.'
        WRITE(9,*)' '
        WRITE(9,*)'S`(',X(0),'  ) = '
        READ(9,*) DX0
        WRITE(9,*)' '
        WRITE(9,*)'S`(',X(N),'  ) = '
        READ(9,*) DXN
        WRITE(9,*)' '
        B(1)=B(1)-H(0)/2
        V(1)=V(1)-3*(D(0)-DX0)
        B(N-1)=B(N-1)-H(N-1)/2
        V(N-1)=V(N-1)-3*(DXN-D(N-1))
      ELSEIF (Stype.EQ.2) THEN
        M(0)=0
        M(N)=0
      ELSEIF (Stype.EQ.3) THEN
        B(1)=B(1)+H(0)+H(0)*H(0)/H(1)
        C(1)=C(1)-H(0)*H(0)/H(1)
        B(N-1)=B(N-1)+H(N-1)+H(N-1)*H(N-1)/H(N-2)
        A(N-2)=A(N-2)-H(N-1)*H(N-1)/H(N-2)
      ELSEIF (Stype.EQ.4) THEN
        B(1)=B(1)+H(0)
        B(N-1)=B(N-1)+H(N-1)
      ELSEIF (Stype.EQ.5) THEN
        WRITE(9,*)' '
        WRITE(9,*)'ENTER THE SECOND DERIVATIVES.'
        WRITE(9,*)' '
        WRITE(9,*)'S``(',X(0),'  ) = '
        READ(9,*) DDX0
        WRITE(9,*)' '
        WRITE(9,*)'S``(',X(N),'  ) = '
        READ(9,*) DDXN
        WRITE(9,*)' '
        V(1)=V(1)-H(0)*DDX0
        V(N-1)=V(N-1)-H(N-1)*DDXN
      ENDIF
      RETURN
      END

      SUBROUTINE XRIDIAG(A,B,C,V,M,N)
      PARAMETER(MaxN=50)
      INTEGER K,N
      REAL A,B,C,M,T,V
      DIMENSION A(0:MaxN),B(0:MaxN),C(0:MaxN)
      DIMENSION M(0:MaxN),V(0:MaxN)
      DO 10 K=2,(N-1)
        T=A(K-1)/B(K-1)
        B(K)=B(K)-T*C(K-1)
        V(K)=V(K)-T*V(K-1)
10    CONTINUE
      M(N-1)=V(N-1)/B(N-1)
      DO 20 K=(N-2),1,-1
        M(K)=(V(K)-C(K)*M(K+1))/B(K)
20    CONTINUE
      RETURN
      END

      SUBROUTINE XNDCOEF(D,H,M,N,Stype,DX0,DXN,DDX0,DDXN)
      PARAMETER(MaxN=50)
      INTEGER N,Stype
      REAL D,DX0,DXN,DDX0,DDXN,H,M
      DIMENSION D(0:MaxN),H(0:MaxN),M(0:MaxN)
      IF (Stype.EQ.1) THEN
        M(0)=3*(D(0)-DX0)/H(0)-M(1)/2
        M(N)=3*(DXN-D(N-1))/H(N-1)-M(N-1)/2
      ELSEIF (Stype.EQ.2) THEN
        M(0)=0
        M(N)=0
      ELSEIF (Stype.EQ.3) THEN
        M(0)=M(1)-H(0)*(M(2)-M(1))/H(1)
        M(N)=M(N-1)+H(N-1)*(M(N-1)-M(N-2))/H(N-2)
      ELSEIF (Stype.EQ.4) THEN
        M(0)=M(1)
        M(N)=M(N-1)
      ELSEIF (Stype.EQ.5) THEN
        M(0)=DDX0
        M(N)=DDXN
      ENDIF
      RETURN
      END

      SUBROUTINE XOMCOEF(D,H,M,S,Y,N)
      PARAMETER(MaxN=50)
      INTEGER I,K,N
      REAL D,H,M,S
      DIMENSION D(0:MaxN),H(0:MaxN),M(0:MaxN),Y(0:MaxN),S(0:MaxN,0:3)
      DO 10 K=0,(N-1)
        S(K,0)=Y(K)
        S(K,1)=D(K)-H(K)*(2*M(K)+M(K+1))/6
        S(K,2)=M(K)/2
        S(K,3)=(M(K+1)-M(K))/(6*H(K))
10    CONTINUE
      RETURN
      END

      REAL FUNCTION XCS(S,X,N,T)
      PARAMETER(MaxN=50)
      INTEGER J,K,N
      REAL S,T,W,X
      DIMENSION X(0:MaxN),S(0:MaxN,0:3)
      K=N-1
      DO 10 J=1,N
        IF (T.LE.X(J)) THEN
          K=J-1
          GOTO 20
        ENDIF
10    CONTINUE
20    CONTINUE
      IF (T.LE.X(0)) THEN
        K=0
      ENDIF
      W=T-X(K)
      CS=((S(K,3)*W + S(K,2))*W + S(K,1))*W + S(K,0)
      RETURN
      END

      SUBROUTINE GETDATA(X,Y,N)
      PARAMETER(MaxN=50)
      INTEGER I,K,N
      REAL X,Y
      DIMENSION X(0:MaxN),Y(0:MaxN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'A CUBIC SPLINE IS CONSTRUCTED WHICH PASSES THROUGH'
      WRITE(9,*)' '
      WRITE(9,*)'THE  N+1  POINTS (X ,Y ), (X ,Y ),..., (X ,Y )'
      WRITE(9,*)'                   0  0     1  1         N  N '
      WRITE(9,*)' '
      WRITE(9,*)'THE SOLUTION IS A SET OF  N  PIECEWISE CUBIC FUNCTIONS'
      WRITE(9,*)' '
      WRITE(9,*)'                   3             2'
      WRITE(9,*)'S (X) =  S   (X-X )  + S   (X-X )  + S   (X-X ) + S'
      WRITE(9,*)' K        K,3    K      K,2    K      K,1    K     K,0'
      WRITE(9,*)' '
      WRITE(9,*)'        WHERE X IS IN [X ,X   ]'
      WRITE(9,*)'                        K  K+1 '
      WRITE(9,*)' '
      WRITE(9,*)'        AND K = 0,1,...,N-1.'
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE NUMBER  N = '
      READ(9,*) N
      WRITE(9,*)' '
      WRITE(9,*)'NOW ENTER THE ',(N+1),' POINTS:'
      WRITE(9,*)' '
      DO 20 K=0,N
        WRITE(9,*)'X(',K,') = '
        READ(9,*) X(K)
        WRITE(9,*)'Y(',K,') = '
        READ(9,*) Y(K)
        WRITE(9,*)' '
20    CONTINUE
      RETURN
      END

      SUBROUTINE SPLTYPE(Stype)
      INTEGER I,Stype
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'WHICH SPLINE TYPE DO YOU WANT? <1-5> '
      WRITE(9,*)' '
      WRITE(9,*)'1 - CLAMPED SPLINE'
      WRITE(9,*)' '
      WRITE(9,*)'2 - NATURAL SPLINE'
      WRITE(9,*)' '
      WRITE(9,*)'3 - EXTRAPOLATE S``(X) NEAR THE ENDPOINTS'
      WRITE(9,*)' '
      WRITE(9,*)'4 - PARABOLIC NEAR THE ENDPOINTS'
      WRITE(9,*)' '
      WRITE(9,*)'5 - SPECIFY  S``(X(0)) AND S``(X(N))'
      WRITE(9,*)' '
      WRITE(9,*)'    CHOOSE TYPE <1-5> '
      READ(9,*) Stype
      RETURN
      END

      SUBROUTINE RESULTS(S,X,Y,N,Stype,DX0,DXN,DDX0,DDXN)
      PARAMETER(MaxN=50)
      INTEGER I,K,N,Stype
      REAL DX0,DXN,DDX0,DDXN,S,X,Y
      DIMENSION X(0:MaxN),Y(0:MaxN),S(0:MaxN,0:3)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)' THE DATA POINTS ARE:'
      WRITE(9,*)' '
      IF (Stype.EQ.1) THEN
        WRITE(9,*)' K        X(K)           Y(K)           S`(X(K))'
      ELSEIF (Stype.EQ.5) THEN
        WRITE(9,*)' K        X(K)           Y(K)           S``(X(K))'
      ELSE
        WRITE(9,*)' K        X(K)           Y(K)'
      ENDIF
      DO 20 K=0,N
      IF (Stype.EQ.1 .AND. K.EQ.0) THEN
        WRITE(9,999) K,X(K),Y(K),DX0
      ELSEIF (Stype.EQ.1 .AND. K.EQ.N) THEN
        WRITE(9,999) K,X(K),Y(K),DXN
      ELSEIF (Stype.EQ.5 .AND. K.EQ.0) THEN
        WRITE(9,999) K,X(K),Y(K),DDX0
      ELSEIF (Stype.EQ.5 .AND. K.EQ.N) THEN
        WRITE(9,999) K,X(K),Y(K),DDXN
      ELSE
        WRITE(9,999) K,X(K),Y(K)
      ENDIF
20    CONTINUE
      WRITE(9,*)' '
      IF (Stype.EQ.1) THEN
        WRITE(9,*)'YOU CHOSE THE CLAMPED SPLINE WITH S`(X(0)) AND S`(X(N
     +)) SPECIFIED.'
      ELSEIF (Stype.EQ.2) THEN
        WRITE(9,*)'YOU CHOSE THE NATURAL SPLINE.'
      ELSEIF (Stype.EQ.3) THEN
        WRITE(9,*)'YOU CHOSE THE SPLINE WITH S``(X) EXTRAPOLATED NEAR TH
     +E ENDPOINTS.'
      ELSEIF (Stype.EQ.4) THEN
        WRITE(9,*)'YOU CHOSE THE SPLINE WHICH IS PARABOLIC NEAR THE ENDP
     +OINTS.'
      ELSEIF (Stype.EQ.5) THEN
        WRITE(9,*)'YOU CHOSE THE SPLINE WITH S``(X(0)) AND S``(X(N)) SPE
     +CIFIED.'
      ENDIF
      WRITE(9,*)' '
      WRITE(9,*)'                         3                 2'
      WRITE(9,*)'S (X)  =  S(K,3)*(X-X(K)) + S(K,2)*(X-X(K)) + S(K,1)*(X
     +-X(K)) + S(K,0)'
      WRITE(9,*)' K'
      WRITE(9,*)' '
      WRITE(9,*)' K        X(K)           S(K,3)         S(K,2)
     +S(K,1)         S(K,0)'
      DO 30 K=0,(N-1)
        WRITE(9,999) K,X(K),S(K,3),S(K,2),S(K,1),S(K,0)
30    CONTINUE
      WRITE(9,*)' '
999   FORMAT(1X,I2,5F15.7)
      RETURN
      END

      SUBROUTINE EVALUAT(CS,S,X,Y,N,Stype,DX0,DXN,DDX0,DDXN)
      PARAMETER(MaxN=50)
      INTEGER N,Stype
      REAL DX0,DXN,DDX0,DDXN,S,T,X,Y,Z
      DIMENSION X(0:MaxN),Y(0:MaxN),S(0:MaxN,0:3)
      EXTERNAL CS
      WRITE(9,*)' '
      WRITE(9,*)'ENTER A VALUE T = '
      READ(9,*) T
      Z=CS(S,X,N,T)
      WRITE(9,*)'THE SPLINE`S VALUE IS  S(',T,'  ) =',Z
      RETURN
      END
