      PROGRAM NUMDERIV
C     ----------------------------------------------------------------
C     Alg6'1.for   FORTRAN program for implementing Algorithm 6.1
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 6.1 (Differentiation Using Limits).
C     Section   6.1, Approximating the Derivative, Page 326
C     ----------------------------------------------------------------
      PARAMETER(Tol=1E-7,Max=100)
      INTEGER M
      REAL DX,H,X
      DIMENSION D(0:Max),DX(0:Max),E(0:Max)
      CHARACTER ANS*1
      EXTERNAL F
10    CALL INPUTS(X)
      CALL DLIMIT(F,D,DX,E,X,H,M,Tol)
      CALL RESULT(D,DX,E,X,H,M)
      WRITE(9,*)' '
      WRITE(9,*) 'WANT TO COMPUTE ANOTHER VALUE OF F`(X) ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
      REAL X
      F=COS(X)
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)'    F(X)  =  COS(X)'
      RETURN
      END

      SUBROUTINE DLIMIT(F,D,DX,E,X,H,M,Tol)
      PARAMETER(Max=100)
      INTEGER M,N
      REAL D,DX,E,H,R,X
      DIMENSION D(0:Max),DX(0:Max),E(0:Max),R(0:Max)
      CHARACTER ANS*1
      EXTERNAL F
      Small=1E-9
      H = 1
      DX(0) = H
      D(0) = 0.5*(F(X+H)-F(X-H))/H
      DO N=1,2
        H = H/2
        DX(N) = H
        D(N) = 0.5*(F(X+H)-F(X-H))/H
        E(N) = ABS(D(N)-D(N-1))
        R(N) = 2*E(N)/(ABS(D(N))+ABS(D(N-1))+Small)
      ENDDO
      N = 1
      WHILE ((E(N).GT.E(N+1) .OR. R(N).GT.Tol).AND.(N.LT.Max))
        H = H/2
        DX(N+2) = H
        D(N+2) = 0.5*(F(X+H)-F(X-H))/H
        E(N+2) = ABS(D(N+2)-D(N+1))
        R(N+2) = 2*E(N+2)/(ABS(D(N+2))+ABS(D(N+1))+Small)
        N = N+1
      REPEAT
      M=N
      RETURN
      END

      SUBROUTINE XLIMIT(F,D,DX,E,X,H,M,Tol)
      PARAMETER(Max=100)
      INTEGER M,N
      REAL D,DX,E,H,R,X
      DIMENSION D(0:Max),DX(0:Max),E(0:Max),R(0:Max)
      CHARACTER ANS*1
      EXTERNAL F
      Small=1E-9
      H = 1
      DX(0) = H
      D(0) = 0.5*(F(X+H)-F(X-H))/H
      DO 10 N=1,2
        H = H/2
        DX(N) = H
        D(N) = 0.5*(F(X+H)-F(X-H))/H
        E(N) = ABS(D(N)-D(N-1))
        R(N) = 2*E(N)/(ABS(D(N))+ABS(D(N-1))+Small)
10    CONTINUE
      N = 1
20    IF ((E(N).GT.E(N+1) .OR. R(N).GT.Tol).AND.(N.LT.Max)) THEN
        H = H/2
        DX(N+2) = H
        D(N+2) = 0.5*(F(X+H)-F(X-H))/H
        E(N+2) = ABS(D(N+2)-D(N+1))
        R(N+2) = 2*E(N+2)/(ABS(D(N+2))+ABS(D(N+1))+Small)
        N = N+1
        GOTO 20
      ENDIF
      M=N
      RETURN
      END

      SUBROUTINE INPUTS(X)
      INTEGER I
      REAL X
      DO 10 I=1,16
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'THE LIMIT OF THE SYMMETRIC DIFFERENCE QUOTIENTS'
      WRITE(9,*)' '
      WRITE(9,*)'IS USED TO FIND AN APPROXIMATION TO  F`(X):'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'   F`(X)  =  LIM  [F(X+H) - F(X-H)]/2H'
      WRITE(9,*)'            H -> 0 '
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'THE GIVEN FUNCTION IS:'
      WRITE(9,*)' '
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE VALUE  X = '
      READ(9,*) X
      RETURN
      END

      SUBROUTINE RESULT(D,DX,E,X,H,M)
      PARAMETER(Max=100)
      INTEGER I,M
      REAL B,D,DL,DR,DX,E,H,X
      DIMENSION D(0:Max),DX(0:Max),E(0:Max)
      EXTERNAL F
      WRITE(9,*)' '
      WRITE(9,*)'          H
     &                      ( F(X+H) - F(X-H) )/2H'
      DO 10 N=0,M-1
      WRITE(9,1000) DX(N),D(N)
10    CONTINUE
      WRITE(9,*)' '
      PAUSE
      WRITE(9,*)' '
      WRITE(9,*)'THE LIMIT OF THE SYMMETRIC DIFFERENCE QUOTIENTS WAS
     + FOUND'
      WRITE(9,*)' '
      WRITE(9,*)'   F`(X)  =  LIM  [F(X+H) - F(X-H)]/2H'
      WRITE(9,*)'            H -> 0 '
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'   AFTER ',M,' ITERATIONS, H =',H
      WRITE(9,*)' '
      WRITE(9,*)'AN APPROXIMATION FOR THE VALUE OF THE DERIVATIVE IS:'
      WRITE(9,*)' '
      WRITE(9,*)'F`(',X,'  )  =',D(M-1)
      WRITE(9,*)' '
      WRITE(9,*)'        THE ACCURACY IS  +-',E(M-1)
      DL=(F(X)-F(X-H))/H
      DR=(F(X+H)-F(X))/H
      B=1+ABS(DL)+ABS(DR)
      IF (ABS(DR-DL)>0.001*B) THEN
        WRITE(9,*)' '
        WRITE(9,*)'THE LEFT-HAND AND RIGHT-HAND DERIVATIVES ARE NOT
     + EQUAL.'
        WRITE(9,*)' '
        WRITE(9,*)'LEFT-HAND   [F(X)-F(X-H)]/H =',DL
        WRITE(9,*)' '
        WRITE(9,*)'RIGHT-HAND  [F(X+H)-F(X)]/H =',DR
        WRITE(9,*)' '
        WRITE(9,*)'PERHAPS F`(X) DOES NOT EXIST.'
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(F20.10,5X,F20.10)
      END
