      PROGRAM DIFFEXTR
C     ----------------------------------------------------------------
C     Alg6'2.for   FORTRAN program for implementing Algorithm 6.2
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 6.2 (Differentiation Using Extrapolation). 
C     Section   6.1, Approximating the Derivative, Page 327
C     ----------------------------------------------------------------
      PARAMETER(Tol=1E-7,Delta=1E-7,MaxN=15)
      INTEGER J,K,N
      REAL D,DX,Error,H,RelErr,X
      DIMENSION D(0:MaxN,0:MaxN),DX(0:MaxN)
      CHARACTER ANS*1
      EXTERNAL F
10    CALL INPUTS(X)
      CALL EXTRAP(F,D,DX,X,H,Error,N,Tol,Delta)
      CALL RESULT(F,D,DX,X,H,Error,N)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO EVALUATE F`(X) AT ANOTHER POINT ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
      REAL X
      F=SIN(X)
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)'   F(X)  =  SIN(X)'
      RETURN
      END

      SUBROUTINE EXTRAP(F,D,DX,X,H,Error,N,Tol,Delta)
      PARAMETER(MaxN=15)
      INTEGER J,K,N
      REAL D,DX,Error,H,RelErr,Small,X
      DIMENSION D(0:MaxN,0:MaxN),DX(0:MaxN)
      EXTERNAL F
      Small=1E-7
      H=1
      DX(0) = H
      J=1
      Error=1
      RelErr=1
      D(0,0)=0.5*(F(X+H)-F(X-H))/H
      WHILE ( RelErr.GT.Tol .AND. Error.GT.Delta .AND. J.LT.16)
        H=H/2
        DX(J) = H
        D(J,0)=0.5*(F(X+H)-F(X-H))/H
        DO K=1,J
          D(J,K)=D(J,K-1)+(D(J,K-1)-D(J-1,K-1))/(4**K-1)
        ENDDO
        Error=ABS(D(J,J)-D(J-1,J-1))
        RelErr=2*Error/(ABS(D(J,J))+ABS(D(J-1,J-1))+Small)
        N=J
        J=J+1
      REPEAT
      RETURN
      END

      SUBROUTINE XXTRAP(F,D,DX,X,H,Error,N,Tol,Delta)
      PARAMETER(MaxN=15)
      INTEGER J,K,N
      REAL D,DX,Error,H,RelErr,Small,X
      DIMENSION D(0:MaxN,0:MaxN),DX(0:MaxN)
      EXTERNAL F
      Small=1E-7
      H=1
      DX(0) = H
      J=1
      Error=1
      RelErr=1
      D(0,0)=0.5*(F(X+H)-F(X-H))/H
10    IF ( RelErr.GT.Tol .AND. Error.GT.Delta .AND. J.LT.16) THEN
      H=H/2
      DX(J) = H
      D(J,0)=0.5*(F(X+H)-F(X-H))/H
      DO 20 K=1,J
        D(J,K)=D(J,K-1)+(D(J,K-1)-D(J-1,K-1))/(4**K-1)
20    CONTINUE
        Error=ABS(D(J,J)-D(J-1,J-1))
        RelErr=2*Error/(ABS(D(J,J))+ABS(D(J-1,J-1))+Small)
        N=J
        J=J+1
        GOTO 10
      ENDIF
      RETURN
      END

      SUBROUTINE INPUTS(X)
      INTEGER I
      REAL X
      DO 10 I=1,18
      WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'RICHARDSON`S EXTRAPOLATION APPLIED TO THE SEQUENCE OF
     + DIFFERENCE QUOTIENTS'
      WRITE(9,*)' '
      WRITE(9,*)'                                                -K  '
      WRITE(9,*)'        { [F(X+H ) - F(X-H )]/2H : WHERE  H  = 2   }'
      WRITE(9,*)'                K         K     K          K'
      WRITE(9,*)' '
      WRITE(9,*)'IS USED TO FIND AN APPROXIMATION TO THE DERIVATIVE OF
     + THE FUNCTION:'
      WRITE(9,*)' '
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE VALUE  X = '
      READ(9,*) X
      RETURN
      END

      SUBROUTINE RESULT(F,D,DX,X,H,Error,N)
      PARAMETER(MaxN=15)
      INTEGER I,N
      REAL B,D,DL,DR,DX,Error,H,H0,X
      DIMENSION D(0:MaxN,0:MaxN),DX(0:MaxN)
      EXTERNAL F
      WRITE(9,*)' '
      WRITE(9,*)'          H                        D(N,N)'
      DO 10 I=0,N
      WRITE(9,1000) DX(I),D(I,I)
10    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'RICHARDSON`S EXTRAPOLATION WAS USED ON THE SEQUENCE OF
     + DIFFERENCE QUOTIENTS'
      WRITE(9,*)'                                                    -K'
      WRITE(9,*)'          { [F(X+H ) - F(X-H )]/2H  : WHERE  H  = 2
     +}'
      WRITE(9,*)'                  K         K     K           K'
      WRITE(9,*)'TO FIND AN APPROXIMATION TO THE DERIVATIVE OF THE
     + FUNCTION:'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'   AFTER ',N,' ITERATIONS, H =',H
      WRITE(9,*)' '
      WRITE(9,*)'AN APPROXIMATION FOR THE VALUE OF THE DERIVATIVE IS'
      WRITE(9,*)' '
      WRITE(9,*)'F`(',X,'  )  =',D(N,N)
      WRITE(9,*)' '
      WRITE(9,*)'        THE ACCURACY IS  +-',Error
      H0=0.002
      IF (H.LT.H0) THEN
        H0=H
      ENDIF
      DL=(F(X)-F(X-H0))/H0
      DR=(F(X+H0)-F(X))/H0
      B=1+ABS(DL)+ABS(DR)
      IF (ABS(DR-DL).GT.0.02*B) THEN
        WRITE(9,*)' '
        WRITE(9,*)'THE LEFT-HAND AND RIGHT-HAND DERIVATIVES ARE NOT
     + EQUAL.'
        WRITE(9,*)' '
        WRITE(9,*)'LEFT-HAND   [F(X)-F(X-H0)]/H0 =',DL
        WRITE(9,*)' '
        WRITE(9,*)'RIGHT-HAND  [F(X+H0)-F(X)]/H0 =',DR
        WRITE(9,*)' '
        WRITE(9,*)'PERHAPS F`(X) DOES NOT EXIST.'
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(F20.10,5X,F20.10)
      END
