      PROGRAM ROMBERG
C     ----------------------------------------------------------------
C     Alg7'4.for   FORTRAN program for implementing Algorithm 7.4
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 7.3 (Recursive Trapezoidal Rule).
C     Algorithm 7.4 (Romberg Integration).
C     Section   7.3, Recursive Rules and Romberg Integration, Page 379
C     ----------------------------------------------------------------
      PARAMETER(MaxN=12,Tol=1E-6)
      INTEGER J
      REAL A,B,Close,R,Rrule
      CHARACTER*56 ANS*1,DFUN,FUN
      DIMENSION R(0:MaxN,0:MaxN)
      EXTERNAL F
10    CALL INPUTS(A,B,DFUN,FUN)
      CALL ROMBRU(F,A,B,Tol,R,Rrule,Close,J)
      CALL RESULT(A,B,R,Rrule,Close,J,DFUN,FUN)
      WRITE(9,*)'WANT TO TRY ANOTHER INTERVAL ? <Y/N> '
      READ (9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      END

      REAL FUNCTION F(X)
      REAL X
      F=SIN(X)
      RETURN
      END

      SUBROUTINE PRINTFUN(DFUN,FUN)
      CHARACTER*(*) DFUN,FUN
      FUN ='SIN(X)'
      DFUN='SIN(X) DX'
      RETURN
      END

      SUBROUTINE ROMBRU(F,A,B,Tol,R,Rrule,Close,J)
      PARAMETER(MaxN=12,Min=4)
      INTEGER J,K,M,P
      REAL A,B,Close,H,R,Rrule,Sum,Tol,X
      DIMENSION R(0:MaxN,0:MaxN)
      EXTERNAL F
      M=1
      H=B-A
      Close=1
      J=0
      R(0,0)=H*(F(A)+F(B))/2
      WHILE ((Close.GT.Tol .AND. J.LT.MaxN) .OR. J.LT.Min)
        J=J+1               !This is the Trapezoidal Rule:
        H=H/2
        Sum=0
        DO P=1,M
          X=A+H*(2*P-1)
          Sum=Sum+F(X)
        ENDDO
        R(J,0)=H*Sum+R(J-1,0)/2
        M=2*M             !This is the Extrapolation part:
        DO K=1,J
          R(J,K)=R(J,K-1)+(R(J,K-1)-R(J-1,K-1))/(4**K-1)
        ENDDO
        Close = ABS(R(J-1,J-1)-R(J,J))
      REPEAT
      Rrule=R(J,J)
      RETURN
      END

      SUBROUTINE XOMBRU(F,A,B,Tol,R,Rrule,Close,J)
      PARAMETER(MaxN=12,Min=4)
      INTEGER J,K,M,P
      REAL A,B,Close,H,R,Rrule,Sum,Tol,X
      DIMENSION R(0:MaxN,0:MaxN)
      EXTERNAL F
      M=1
      H=B-A
      Close=1
      J=0
      R(0,0)=H*(F(A)+F(B))/2
10    IF ((Close.GT.Tol .AND. J.LT.MaxN) .OR. J.LT.Min) THEN
        J=J+1                 !This is the Trapezoidal Rule:
        H=H/2
        Sum=0
        DO 20 P=1,M
          X=A+H*(2*P-1)
          Sum=Sum+F(X)
20      CONTINUE
        R(J,0)=H*Sum+R(J-1,0)/2
        M=2*M                !This is the Extrapolation part:
        DO 30 K=1,J
          R(J,K)=R(J,K-1)+(R(J,K-1)-R(J-1,K-1))/(4**K-1)
30      CONTINUE
        Close = ABS(R(J-1,J-1)-R(J,J))
        GOTO 10
      ENDIF
      Rrule=R(J,J)
      RETURN
      END

      SUBROUTINE INPUTS(A,B,DFUN,FUN)
      INTEGER I
      REAL A,B
      CHARACTER*(*) DFUN,FUN
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'    ROMBERG INTEGRATION IS PERFORMED TO FIND AN APPROXI
     +MATION'
      WRITE(9,*)' '
      WRITE(9,*)'FOR THE VALUE OF THE DEFINITE INTEGRAL:'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'    B'
      WRITE(9,*)'    /'
      WRITE(9,*)'    | ',DFUN
      WRITE(9,*)'    /'
      WRITE(9,*)'    A'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE LEFT  ENDPOINT A = '
      READ(9,*) A
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE RIGHT ENDPOINT B = '
      READ(9,*) B
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(A,B,R,Rrule,Close,J,DFUN,FUN)
      PARAMETER(MaxN=12,Tol=1E-9)
      INTEGER I,J,K
      REAL A,B,Close,R,Rrule
      CHARACTER*(*) DFUN,FUN
      DIMENSION R(0:MaxN,0:MaxN)
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'           ',B
      WRITE(9,*)'              /'
      WRITE(9,*)'              |'
      WRITE(9,*)Rrule,'  ~   |  ',DFUN
      WRITE(9,*)'              |'
      WRITE(9,*)'              /'
      WRITE(9,*)'           ',A
      WRITE(9,*)' '
      WRITE(9,*)'THE DIAGONAL ELEMENTS IN THE ROMBERG TABLE ARE:'
      IF (J.LE.5) THEN
        WRITE(9,*)' '
      ENDIF
      DO 20 K=0,J,2
        IF (K.LT.J) THEN
          WRITE(9,998) K,K,R(K,K),K+1,K+1,R(K+1,K+1)
        ELSE
          WRITE(9,999) K,K,R(K,K)
        ENDIF
20    CONTINUE
      IF (J.LE.7) THEN
        WRITE(9,*)' '
      ENDIF
      WRITE(9,*)'AN APPROXIMATE VALUE FOR THE DEFINITE INTEGRAL OF'
      WRITE(9,*)' '
      WRITE(9,*)'F(X) = ',FUN
      WRITE(9,*)' '
      WRITE(9,*)'TAKEN OVER  [',A,'  ,',B,'  ]  WAS FOUND.'
      WRITE(9,*)' '
      WRITE(9,*)'THE ROMBERG APPROXIMATION IS ',Rrule
      IF (J.LE.9) THEN
        WRITE(9,*)' '
      ENDIF
      WRITE(9,*)'    THE  ERROR  ESTIMATE  IS ',Close
      IF (J.LE.11) THEN
        WRITE(9,*)' '
      ENDIF
998   FORMAT(' R(',I2,',',I2,') = ',F15.7,
     +'        R(',I2,',',I2,') = ',F15.7)
999   FORMAT(' R(',I2,',',I2,') = ',F15.7)
      RETURN
      END
