      PROGRAM ADAQUAD
C     ----------------------------------------------------------------
C     Alg7'5.for   FORTRAN program for implementing Algorithm 7.5
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 7.5 (Adaptive Quadrature Using Simpson's Rule).
C     Section   7.4, Adaptive Quadrature, Page 389
C     ----------------------------------------------------------------
      PARAMETER(MaxN=12,Tol=1E-6)
      INTEGER P,M,State
      REAL A,B,Tol0,SRmat,SRvec,Integral,ErrBdd
      REAL Close,Rrule
      INTEGER J
      CHARACTER*56 ANS*1,DFUN,FUN
      DIMENSION SRvec(1:11),SRmat(1:101,1:11)
      EXTERNAL F
10    CALL INPUTS(A,B,DFUN,FUN)
      CALL AdaptQuad(F,A,B,Tol,SRmat,Integral,ErrBdd,M,State)
      CALL RESULT(A,B,SRmat,Integral,ErrBdd,M,DFUN,FUN)
      WRITE(9,*)'WANT TO TRY ANOTHER INTERVAL ? <Y/N> '
      READ (9,'(A)') ANS
      IF (ANS.NE.'Y' .AND. ANS.NE.'y') STOP
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
      REAL X
      F=SIN(X)
      RETURN
      END

      SUBROUTINE PRINTFUN(DFUN,FUN)
      CHARACTER*(*) DFUN,FUN
      FUN ='SIN(X)'
      DFUN='SIN(X) DX'
      RETURN
      END

      SUBROUTINE AdaptQuad(F,A,B,Tol,SRmat,Integral,ErrBdd,M,State)
      INTEGER M,State
      REAL A,B,Tol,SRmat,Integral,ErrBdd
      INTEGER J,K,N,Iterating,Done
      REAL Sum1,Sum2,SRvec
      DIMENSION SRmat(1:101,1:11),SRvec(1:11)
      EXTERNAL F
      Iterating = 0
      Done = 1
      CALL Srule(F,A,B,Tol,SRvec)
      DO K=1,11
        SRmat(1, K) = SRvec(K)
      ENDDO
      M = 1
      State = Iterating
      WHILE (State .EQ. Iterating)
        N = M
        DO J=N,1,-1
    CALL Refine(J,SRmat,M,State)
        ENDDO
      REPEAT
      Sum1 = 0
      Sum2 = 0
      DO J=1,M
        Sum1 = Sum1 + SRmat(J, 8)
        Sum2 = Sum2 + Abs(SRmat(J, 9))
      ENDDO
      Integral = Sum1
      ErrBdd = Sum2
      RETURN
      END

      SUBROUTINE Refine(P,SRmat,M,State)
      INTEGER P,M,State
      REAL SRmat
      INTEGER J,K,Iterating,Done
      REAL A,B,C,Err,Fa,Fb,Fc,S,S2,Tol,Tol2,Err,Check
      REAL SR0vec,SR1vec,SR2vec
      DIMENSION SRmat(1:101,1:11)
      DIMENSION SR0vec(1:11),SR1vec(1:11),SR2vec(1:11)
      EXTERNAL F
      Iterating = 0
      Done = 1
      State = Done
      DO K=1,11
        SR0vec(K) = SRmat(P, K)
      ENDDO
      A = SR0vec(1)
      C = SR0vec(2)
      B = SR0vec(3)
      Fa = SR0vec(4)
      Fc = SR0vec(5)
      Fb = SR0vec(6)
      S = SR0vec(7)
      S2 = SR0vec(8)
      Err = SR0vec(9)
      Tol = SR0vec(10)
      Check = SR0vec(11)
      IF (Check .EQ. 1) RETURN
      Tol2 = Tol / 2
      CALL Srule(F, A, C, Tol2, SR1vec)
      CALL Srule(F, C, B, Tol2, SR2vec)
      Err = ABS(SR0vec(7) - SR1vec(7) - SR2vec(7)) / 10
      IF (Err .LT. Tol) THEN
        SR0vec(11) = 1
      ENDIF
      IF (Err .LT. Tol) THEN
        DO K=1,11
          SRmat(P, K) = SR0vec(K)
        ENDDO
        SRmat(P, 8) = SR1vec(7) + SR2vec(7)
        SRmat(P, 9) = Err
      ELSE
        DO J=(M + 1),P,-1
          DO K=1,11
            SRmat(J, K) = SRmat(J - 1, K)
          ENDDO
        ENDDO
        M = M + 1
        DO K=1,11
          SRmat(P, K) = SR1vec(K)
        ENDDO
        DO K=1,11
          SRmat(P + 1, K) = SR2vec(K)
        ENDDO
        State = Iterating
      ENDIF
      RETURN
      END

      SUBROUTINE Srule(F,A,B,Tol0,SRvec)
      REAL A,B,Tol0,SRvec
      REAL C,H,Fa,Fb,Fc,S,S2,Tol1,Err,Check
      DIMENSION SRvec(1:11)
      EXTERNAL F
        H = (B - A) / 2
        C = (A + B) / 2
        Fa = F(A)
        Fc = F(C)
        Fb = F(B)
        S = H * (F(A) + 4 * F(C) + F(B)) / 3
        S2 = S
        Tol1 = Tol0
        Err = Tol0
        Check = 0
        SRvec(1) = A
        SRvec(2) = C
        SRvec(3) = B
        SRvec(4) = Fa
        SRvec(5) = Fc
        SRvec(6) = Fb
        SRvec(7) = S
        SRvec(8) = S2
        SRvec(9) = Err
        SRvec(10) = Tol1
        SRvec(11) = Check
      RETURN
      END

      SUBROUTINE INPUTS(A,B,DFUN,FUN)
      INTEGER I
      REAL A,B
      CHARACTER*(*) DFUN,FUN
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'    ROMBERG INTEGRATION IS PERFORMED TO FIND AN APPROXI
     +MATION'
      WRITE(9,*)' '
      WRITE(9,*)'FOR THE VALUE OF THE DEFINITE INTEGRAL:'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'    B'
      WRITE(9,*)'    /'
      WRITE(9,*)'    | ',DFUN
      WRITE(9,*)'    /'
      WRITE(9,*)'    A'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE LEFT  ENDPOINT A = '
      READ(9,*) A
      WRITE(9,*)' '
      WRITE(9,*)'ENTER THE RIGHT ENDPOINT B = '
      READ(9,*) B
      WRITE(9,*)' '
      RETURN
      END

      SUBROUTINE RESULT(A,B,SRmat,Integral,ErrBdd,M,DFUN,FUN)
      INTEGER I,J,K,M
      REAL A,B,SRmat,Integral,ErrBdd
      CHARACTER*(*) DFUN,FUN
      DIMENSION SRmat(1:101,1:11)
      CALL PRINTFUN(DFUN,FUN)
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'           ',B
      WRITE(9,*)'              /'
      WRITE(9,*)'              |'
      WRITE(9,*)Integral,'  ~   |  ',DFUN
      WRITE(9,*)'              |'
      WRITE(9,*)'              /'
      WRITE(9,*)'           ',A
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'AN APPROXIMATE VALUE FOR THE DEFINITE INTEGRAL OF'
      WRITE(9,*)' '
      WRITE(9,*)'F(X) = ',FUN
      WRITE(9,*)' '
      WRITE(9,*)'TAKEN OVER  [',A,'  ,',B,'  ]  WAS FOUND.'
      WRITE(9,*)' '
      WRITE(9,*)'THE ADAVTIVE QUADRATURE APPROXIMATION IS ',Integral
      WRITE(9,*)' '
      WRITE(9,*)'                THE  ERROR  ESTIMATE  IS ',ErrBdd
      WRITE(9,*)' '
      WRITE(9,*)'     THE NUMBER OF SUBINTERVALS USED WAS ',M
      WRITE(9,*)' '
      RETURN
      END
