      PROGRAM GOLDSEAR
C     ----------------------------------------------------------------
C     Alg8'1.for   FORTRAN program for implementing Algorithm 8.1
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 8.1 (Golden Search for a Minimum).
C     Section   8.1, Minimization of a Function, Page 413
C     ----------------------------------------------------------------
      PARAMETER(Delta=1E-5,Epsilon=1E-5)
      INTEGER K
      REAL A,A0,B,B0,P
      CHARACTER ANS*1
      EXTERNAL F
10    CALL INPUTS(A0,B0,A,B)
      CALL SEARCH(F,A,B,P,K,Delta,Epsilon)
      CALL RESULT(F,A0,B0,A,B,P,Delta,Epsilon,K)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO FIND THE MINIMUM ON ANOTHER INTERVAL ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      STOP
      END

      REAL FUNCTION F(X)
      REAL X
      F=X*X-SIN(X)
      RETURN
      END

      SUBROUTINE PRINTFUN
      WRITE(9,*)'F(X)  =  X*X - SIN(X)'
      RETURN
      END

      SUBROUTINE SEARCH(F,A,B,P,K,Delta,Epsilon)
      INTEGER K
      REAL A,B,C,D,Delta,Epsilon,H,P,Rone,Rtwo,YP,YA,YB,YC,YD
      EXTERNAL F
      WRITE(9,*) '               A                C                B'
      Rone=(SQRT(5.0)-1)/2
      Rtwo=Rone*Rone
      H=B-A
      YA=F(A)
      YB=F(B)
      C=A+Rtwo*H
      D=A+Rone*H
      YC=F(C)
      YD=F(D)
      K=1
      WHILE (ABS(YB-YA).GT.Epsilon .OR. H.GT.Delta)
        IF (YC.LT.YD) THEN
          B=D
          YB=YD
          D=C
          YD=YC
          H=B-A
          C=A+Rtwo*H
          YC=F(C)
        ELSE
          A=C
          YA=YC
          C=D
          YC=YD
          H=B-A
          D=A+Rone*H
          YD=F(D)
        ENDIF
        K=K+1
        WRITE(9,1000) A,C,B
      REPEAT
      P=A
      YP=YA
      IF (YB.LT.YA) THEN
        P=B
        YP=YB
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(5X,3F17.7)
      END

      SUBROUTINE XSEARCH(F,A,B,P,K,Delta,Epsilon)
      INTEGER K
      REAL A,B,C,D,Delta,Epsilon,H,P,Rone,Rtwo,YP,YA,YB,YC,YD
      EXTERNAL F
      WRITE(9,*) '               A                C                B'
      Rone=(SQRT(5.0)-1)/2
      Rtwo=Rone*Rone
      H=B-A
      YA=F(A)
      YB=F(B)
      C=A+Rtwo*H
      D=A+Rone*H
      YC=F(C)
      YD=F(D)
      K=1
10    IF (ABS(YB-YA).GT.Epsilon .OR. H.GT.Delta) THEN
        IF (YC.LT.YD) THEN
          B=D
          YB=YD
          D=C
          YD=YC
          H=B-A
          C=A+Rtwo*H
          YC=F(C)
        ELSE
          A=C
          YA=YC
          C=D
          YC=YD
          H=B-A
          D=A+Rone*H
          YD=F(D)
        ENDIF
        K=K+1
        WRITE(9,1000) A,C,B
        GOTO 10
      ENDIF
      P=A
      YP=YA
      IF (YB.LT.YA) THEN
        P=B
        YP=YB
      ENDIF
      PAUSE
      RETURN
1000  FORMAT(5X,3F17.7)
      END

      SUBROUTINE INPUTS(A0,B0,A,B)
      INTEGER I
      REAL A0,B0,A,B
      DO 10 I=1,16
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'    GOLDEN SEARCH FOR FINDING THE MINIMUM OF THE'
      WRITE(9,*)' '
      WRITE(9,*)'UNIMODAL FUNCTION  F(X)  OVER THE INTERVAL  [A,B].'
      WRITE(9,*)' '
      WRITE(9,*)'SEQUENCES {A } AND {B } CONSTRUCTED, WHICH BRACKET
     + THE MINIMUM,'
      WRITE(9,*)'            K        K'
      WRITE(9,*)' '
      WRITE(9,*)'I.E.  A  < X   < B .  TERMINATION OCCURS WHEN  B - A  <
     + DELTA.'
      WRITE(9,*)'        K   MIN   K                             N   N'
      WRITE(9,*)' '
      WRITE(9,*)'THE GIVEN FUNCTION F(X) IS:'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'ENTER LEFT  ENDPOINT  A = '
      READ(9,*) A
      A0=A
      WRITE(9,*)' '
      WRITE(9,*)'ENTER RIGHT ENDPOINT  B = '
      READ(9,*) B
      B0=B
      RETURN
      END

      SUBROUTINE RESULT(F,A0,B0,A,B,P,Delta,Epsilon,K)
      INTEGER I,K
      REAL A0,B0,A,B,H,Delta,Epsilon,P
      EXTERNAL F
      WRITE(9,*)' '
      WRITE(9,*)'     THE GOLDEN SEARCH WAS USED TO FIND THE MINIMUM'
      WRITE(9,*)' '
      WRITE(9,*)'OF THE FUNCTION  F(X)  OVER THE INTERVAL  [A,B].'
      WRITE(9,*)' '
      WRITE(9,*)' '
      WRITE(9,*)'     IT TOOK ',K,' ITERATIONS TO MINIMIZE'
      WRITE(9,*)' '
      CALL PRINTFUN
      WRITE(9,*)' '
      WRITE(9,*)'OVER THE INTERVAL  [',A0,',',B0,'  ].'
      WRITE(9,*)' '
      WRITE(9,*)'THE  MINIMUM  VALUE F(P)'
      WRITE(9,*)' '
      WRITE(9,*)'          OCCURS  AT  P  = ',P
      WRITE(9,*)' '
      H=B-A
      WRITE(9,*)'    THE  ACCURACY  IS    +-',H
      WRITE(9,*)' '
      WRITE(9,*)'THE FUNCTION VALUE IS:'
      WRITE(9,*)' '
      WRITE(9,*)'F(',P,'  )  = ',F(P)
      RETURN
      END
