      PROGRAM GRADMETH
C     ----------------------------------------------------------------
C     Alg8'4.for   FORTRAN program for implementing Algorithm 8.4
C     
C     NUMERICAL METHODS: FORTRAN Programs, (c) John H. Mathews 1995
C     To accompany the text:
C     NUMERICAL METHODS for Math., Science & Engineering, 2nd Ed, 1992
C     Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
C     Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
C     Prentice Hall, International Editions:   ISBN 0-13-625047-5
C     This free software is compliments of the author.
C     E-mail address:       in%"mathews@fullerton.edu"
C     
C     Algorithm 8.4 (Steepest Descent  or Gradient Method).
C     Section   8.1, Minimization of a Function, Page 418
C     ----------------------------------------------------------------
      PARAMETER(MaxN=6,Jmax=200,Max=50,Delta=1E-5,Epsilon=1E-7)
      INTEGER Cond,Count,N
      REAL Error,H,P0,P1,P2,Pmin,S,Y0,Ymin
      CHARACTER ANS*1
      DIMENSION P0(1:MaxN),P1(1:MaxN),P2(1:MaxN),Pmin(1:MaxN),S(1:MaxN)
      EXTERNAL F
10    CALL INPUTS(P0,Y0,N)
      CALL GRADSR(F,Y0,P0,P1,P2,Pmin,S,N,Jmax,Max,
     +Delta,Epsilon,H,Ymin,Error,Cond,Count)
      CALL RESULT(P0,Pmin,N,H,Ymin,Error,Epsilon,Cond,Count)
      WRITE(9,*)' '
      WRITE(9,*)'WANT TO TRY ANOTHER STARTING POINT ? '
      READ(9,'(A)') ANS
      IF (ANS.EQ.'Y' .OR. ANS.EQ.'y') GOTO 10
      END

      REAL FUNCTION F(P,N)
      PARAMETER(MaxN=6)
      INTEGER N
      REAL P,X,Y,Z,U,V,W
      DIMENSION P(1:MaxN)
      X=P(1)
      Y=P(2)
      Z=P(3)
      U=P(4)
      V=P(5)
      W=P(6)
      F=X*X-4*X+Y*Y-Y-X*Y
      RETURN
      END

      SUBROUTINE PRINTFUN(N)
      INTEGER K,N
      CHARACTER C*2,FUN*80,L*6,R*4
      DIMENSION C(1:6)
        N = 2
      FUN ='X*X-4*X+Y*Y-Y-X*Y'
      C(1)='(X'
      C(2)=',Y'
      C(3)=',Z'
      C(4)=',U'
      C(5)=',V'
      C(6)=',W,'
        L ='     F'
        R =') = '
      WRITE(9,999) L,C(1),(C(K),K=2,N),R,FUN
999   FORMAT(9A)
      RETURN
      END

      SUBROUTINE GRADVECT(P,S,N)
      PARAMETER(MaxN=6)
      INTEGER K,N
      REAL G,Length,P,S,Sum,X,Y,Z,U,V,W
      DIMENSION G(1:MaxN),P(1:MaxN),S(1:MaxN)
        X=P(1)
        Y=P(2)
        Z=P(3)
        U=P(4)
        V=P(5)
        W=P(6)
        G(1)=2*X-4-Y
        G(2)=2*Y-1-X
        Sum=0
        DO K=1,N
          Sum=Sum+G(K)*G(K)
        ENDDO
        Length=SQRT(Sum)
        IF (Length.EQ.0) THEN
          Length=1
        ENDIF
        DO K=1,N
          G(K)=-G(K)
          S(K)=G(K)/Length
        ENDDO
        RETURN
      END

      SUBROUTINE QMIN(F,P0,Y0,N,Jmax,Delta,Epsilon,
     &                P1,P2,Pmin,S,H,Ymin,Error,Cond,Count)
      PARAMETER(MaxN=6)
      INTEGER Cond,Count,J,Jmax,K,N
      REAL Delta,Epsilon,Error,H,P0,P1,P2,Pmin,S,Y0,Ymin
      REAL D,E0,E1,E2,H0,H1,H2,Hmin,Length,Y1,Y2
      DIMENSION P0(1:MaxN),P1(1:MaxN),P2(1:MaxN),Pmin(1:MaxN),S(1:MaxN)
      EXTERNAL F
      PARAMETER (Big=1E8)
      DO K=1,N
        P1(K)=P0(K)+H*S(K)
        P2(K)=P0(K)+2*H*S(K)
      ENDDO
      Y1=F(P1,N)
      Y2=F(P2,N)
      Cond=0
      J=0
      WHILE (J.LT.Jmax .AND. Cond.EQ.0)
        Sum=0
        DO K=1,N
          Sum=Sum+P0(K)*P0(K)
        ENDDO
        Length=SQRT(Sum)
        IF (Y0.LE.Y1) THEN
          DO K=1,N
            P2(K)=P1(K)
          ENDDO
          Y2=Y1
          H=H/2
          DO K=1,N
            P1(K)=P0(K)+H*S(K)
          ENDDO
          Y1=F(P1,N)
        ELSE
          IF (Y2.LE.Y1) THEN
            DO K=1,N
              P1(K)=P2(K)
            ENDDO
            Y1=Y2
            H=2*H
            DO K=1,N
              P2(K)=P0(K)+2*H*S(K)
            ENDDO
            Y2=F(P2,N)
          ELSE
              Cond=-1
          ENDIF
        ENDIF
        J=J+1
        IF (H.LT.Delta) Cond=1
        IF (ABS(H).GT.Big .OR. Length.GT.Big) Cond=5
      REPEAT
      Count=Count+J
      IF (Cond.EQ.5) THEN
        DO K=1,N
          Pmin(K)=P1(K)
        ENDDO
        Ymin=Y1
      ELSE
        D=4*Y1-2*Y0-2*Y2             !Start of a long block:
        IF (D.LT.0) THEN
          Hmin=H*(4*Y1-3*Y0-Y2)/D
        ELSE
          Cond=4
          Hmin=H/3
        ENDIF
        DO K=1,N
          Pmin(K)=P0(K)+Hmin*S(K)
        ENDDO
        Ymin=F(Pmin,N)
        H0=ABS(Hmin)
        H1=ABS(Hmin-H)
        H2=ABS(Hmin-2*H)
        IF (H0.LT.H) H=H0
        IF (H1.LT.H) H=H1
        IF (H2.LT.H) H=H2
        IF (H.EQ.0)  H=Hmin
        IF (H.LT.Delta) Cond=1
        E0=ABS(Y0-Ymin)
        E1=ABS(Y1-Ymin)
        E2=ABS(Y2-Ymin)
        IF (E0.NE.0 .AND. E0.LT.Error) Error=E0
        IF (E1.NE.0 .AND. E1.LT.Error) Error=E1
        IF (E2.NE.0 .AND. E2.LT.Error) Error=E2
        IF (E0.EQ.0 .AND. E1.EQ.0 .AND. E2.EQ.0) Error=0
        IF (Error.LT.Epsilon) Cond=2
        IF (Cond.EQ.2 .AND. H.LT.Delta) Cond=3
      ENDIF      !End of the long block.
      RETURN
      END

      SUBROUTINE GRADSR(F,Y0,P0,P1,P2,Pmin,S,N,Jmax,Max,
     &                  Delta,Epsilon,H,Ymin,Error,Cond,Count)
      PARAMETER(MaxN=6)
      INTEGER Cond,Count,Jmax,K,Max,N
      REAL Delta,Epsilon,Error,H,Length,P0,P1,P2,Pmin,S,Y0,Ymin
      DIMENSION P0(1:MaxN),P1(1:MaxN),P2(1:MaxN),Pmin(1:MaxN),S(1:MaxN)
      EXTERNAL F
      H=1
      Sum=0
      DO K=1,N
        Sum=Sum+P0(K)*P0(K)
      ENDDO
      Length=SQRT(Sum)
      IF (Length.GT.1E4) THEN
        H=Length/1E4
      ENDIF
      Error=1
      Count=0
      Cond=0
      WHILE ( Count.LT.Max .AND. Cond.NE.5 .AND.
     &   (H.GT.Delta .OR. Error.GT.Epsilon) )
        CALL GRADVECT(P0,S,N)
        CALL QMIN(F,P0,Y0,N,Jmax,Delta,Epsilon,
     &            P1,P2,Pmin,S,H,Ymin,Error,Cond,Count)
        DO K=1,N
          P0(K)=Pmin(K)
        ENDDO
        Y0=Ymin
        Count=Count+1
      REPEAT
      RETURN
      END

      SUBROUTINE XRADVECT(P,S,N)
      PARAMETER(MaxN=6)
      INTEGER K,N
      REAL G,Length,P,S,Sum,X,Y,Z,U,V,W
      DIMENSION G(1:MaxN),P(1:MaxN),S(1:MaxN)
        X=P(1)
        Y=P(2)
        Z=P(3)
        U=P(4)
        V=P(5)
        W=P(6)
        G(1)=2*X-4-Y
        G(2)=2*Y-1-X
        Sum=0
        DO 10 K=1,N
          Sum=Sum+G(K)*G(K)
10      CONTINUE
        Length=SQRT(Sum)
        IF (Length.EQ.0) THEN
          Length=1
        ENDIF
        DO 20 K=1,N
          G(K)=-G(K)
          S(K)=G(K)/Length
20      CONTINUE
        RETURN
      END

      SUBROUTINE XQMIN(F,P0,Y0,N,Jmax,Delta,Epsilon,
     +P1,P2,Pmin,S,H,Ymin,Error,Cond,Count)
      PARAMETER(MaxN=6)
      INTEGER Cond,Count,J,Jmax,K,N
      REAL Delta,Epsilon,Error,H,P0,P1,P2,Pmin,S,Y0,Ymin
      REAL D,E0,E1,E2,H0,H1,H2,Hmin,Length,Y1,Y2
      DIMENSION P0(1:MaxN),P1(1:MaxN),P2(1:MaxN),Pmin(1:MaxN),S(1:MaxN)
      EXTERNAL F
      PARAMETER (Big=1E8)
      DO 10 K=1,N
        P1(K)=P0(K)+H*S(K)
        P2(K)=P0(K)+2*H*S(K)
10    CONTINUE
      Y1=F(P1,N)
      Y2=F(P2,N)
      Cond=0
      J=0
20    IF (J.LT.Jmax .AND. Cond.EQ.0) THEN
        Sum=0
        DO 30 K=1,N
          Sum=Sum+P0(K)*P0(K)
30      CONTINUE
        Length=SQRT(Sum)
        IF (Y0.LE.Y1) THEN
          DO 40 K=1,N
            P2(K)=P1(K)
40        CONTINUE
          Y2=Y1
          H=H/2
          DO 50 K=1,N
            P1(K)=P0(K)+H*S(K)
50        CONTINUE
          Y1=F(P1,N)
        ELSE
          IF (Y2.LE.Y1) THEN
            DO 60 K=1,N
              P1(K)=P2(K)
60          CONTINUE
            Y1=Y2
            H=2*H
            DO 70 K=1,N
              P2(K)=P0(K)+2*H*S(K)
70          CONTINUE
            Y2=F(P2,N)
          ELSE
              Cond=-1
          ENDIF
        ENDIF
        J=J+1
        IF (H.LT.Delta) Cond=1
        IF (ABS(H).GT.Big .OR. Length.GT.Big) Cond=5
        GOTO 20
      ENDIF
      Count=Count+J
      IF (Cond.EQ.5) THEN
        DO 80 K=1,N
          Pmin(K)=P1(K)
80      CONTINUE
        Ymin=Y1
      ELSE
        D=4*Y1-2*Y0-2*Y2             !Start of a long block:
        IF (D.LT.0) THEN
          Hmin=H*(4*Y1-3*Y0-Y2)/D
        ELSE
          Cond=4
          Hmin=H/3
        ENDIF
        DO 90 K=1,N
          Pmin(K)=P0(K)+Hmin*S(K)
90      CONTINUE
        Ymin=F(Pmin,N)
        H0=ABS(Hmin)
        H1=ABS(Hmin-H)
        H2=ABS(Hmin-2*H)
        IF (H0.LT.H) H=H0
        IF (H1.LT.H) H=H1
        IF (H2.LT.H) H=H2
        IF (H.EQ.0)  H=Hmin
        IF (H.LT.Delta) Cond=1
        E0=ABS(Y0-Ymin)
        E1=ABS(Y1-Ymin)
        E2=ABS(Y2-Ymin)
        IF (E0.NE.0 .AND. E0.LT.Error) Error=E0
        IF (E1.NE.0 .AND. E1.LT.Error) Error=E1
        IF (E2.NE.0 .AND. E2.LT.Error) Error=E2
        IF (E0.EQ.0 .AND. E1.EQ.0 .AND. E2.EQ.0) Error=0
        IF (Error.LT.Epsilon) Cond=2
        IF (Cond.EQ.2 .AND. H.LT.Delta) Cond=3
      ENDIF      !End of the long block.
      RETURN
      END

      SUBROUTINE XGRADSR(F,Y0,P0,P1,P2,Pmin,S,N,Jmax,Max,
     +Delta,Epsilon,H,Ymin,Error,Cond,Count)
      PARAMETER(MaxN=6)
      INTEGER Cond,Count,Jmax,K,Max,N
      REAL Delta,Epsilon,Error,H,Length,P0,P1,P2,Pmin,S,Y0,Ymin
      DIMENSION P0(1:MaxN),P1(1:MaxN),P2(1:MaxN),Pmin(1:MaxN),S(1:MaxN)
      EXTERNAL F
      H=1
      Sum=0
      DO 10 K=1,N
        Sum=Sum+P0(K)*P0(K)
10    CONTINUE
      Length=SQRT(Sum)
      IF (Length.GT.1E4) THEN
        H=Length/1E4
      ENDIF
      Error=1
      Count=0
      Cond=0
20    IF ( Count.LT.Max .AND. Cond.NE.5 .AND.
     &   (H.GT.Delta .OR. Error.GT.Epsilon) ) THEN
        CALL GRADVECT(P0,S,N)
        CALL QMIN(F,P0,Y0,N,Jmax,Delta,Epsilon,
     &            P1,P2,Pmin,S,H,Ymin,Error,Cond,Count)
        DO 30 K=1,N
          P0(K)=Pmin(K)
30      CONTINUE
        Y0=Ymin
        Count=Count+1
        GOTO 20
      ENDIF
      RETURN
      END

      SUBROUTINE INPUTS(P0,Y0,N)
      PARAMETER(MaxN=6)
      INTEGER I,J,K,N
      REAL P0,Y0,Z
      CHARACTER RESP*40
      DIMENSION P0(1:MaxN),Z(1:MaxN)
      EXTERNAL F
      DO 10 I=1,18
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'   THE GRADIENT METHOD OR "METHOD OF STEEPEST DESCENT" I
     +S USED FOR FINDING'
      WRITE(9,*)' '
      WRITE(9,*)'THE MINIMUM OF THE FUNCTION  F(P)  WHERE  P = (p ,p ,..
     +.,p )  AND  N<=6.'
      WRITE(9,*)'                                                1  2
     +   N'
      WRITE(9,*)' '
      WRITE(9,*)'FOR CONVENIENCE, THE FUNCTION F(P) CAN BE EXPRESSED USI
     +NG THE VARIABLES'
      WRITE(9,*)' '
      WRITE(9,*)'X = p , Y = p , Z = p , U = p , V = p , W = p .'
      WRITE(9,*)'     1       2       3       4       5       6 '
      WRITE(9,*)' '
      WRITE(9,*)'    THE FUNCTION TO BE MINIMIZED IS:'
      WRITE(9,*)' '
      CALL PRINTFUN(N)
      WRITE(9,*)' '
      IF (N.EQ.2) THEN
        WRITE(9,*)'GIVE THE STARTING POINT P = (p ,p )'
        WRITE(9,*)'                              1  2'
      ELSEIF (N.EQ.3) THEN
        WRITE(9,*)'GIVE THE STARTING POINT P = (p ,p ,p )'
        WRITE(9,*)'                              1  2  3'
      ELSE
        WRITE(9,*)'GIVE THE STARTING POINT P = (p ,p ,...,p )'
        WRITE(9,*)'                              1  2      N'
        WRITE(9,*)'WHERE N=',N
      ENDIF
      WRITE(9,*)' '
      DO 20 K=1,N
        WRITE(9,*)'P(',K,') = '
        READ(9,*) P0(K)
20    CONTINUE
      Y0=F(P0,N)
      RETURN
      END

      SUBROUTINE RESULT(P0,Pmin,N,H,Ymin,Error,Epsilon,Cond,Count)
      PARAMETER(MaxN=6)
      INTEGER Cond,Count,I,K,N
      REAL Epsilon,Error,H,P0,Pmin,Ymin
      DIMENSION P0(1:MaxN),Pmin(1:MaxN)
      EXTERNAL F
      DO 10 I=1,16
        WRITE(9,*)' '
10    CONTINUE
      WRITE(9,*)'   THE GRADIENT SEARCH METHOD WAS USED TO FIND THE MINI
     +MUM OF'
      WRITE(9,*)' '
      CALL PRINTFUN(N)
      WRITE(9,*) Count,' ITERATIONS WERE REQUIRED'
      WRITE(9,*)' '
      DO 20 K=1,N
        WRITE(9,*)'P(',K,')  = ',Pmin(K)
20    CONTINUE
      WRITE(9,*)' '
      WRITE(9,*)'  DP  = ',H   ,'  IS AN ESTIMATE FOR THE ACCURACY.'
      WRITE(9,*)' '
      WRITE(9,*)'THE MINIMUM VALUE OF THE FUNCTION IS'
      WRITE(9,*)' '
      WRITE(9,*)'F(P)  = ',Ymin
      WRITE(9,*)' '
      WRITE(9,*)'  DF  = ',Error,'  IS AN ESTIMATE FOR THE ACCURACY.'
      WRITE(9,*)' '
      IF (Cond.EQ.0) THEN
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL BECAUSE THE'
        WRITE(9,*)'MAXIMUM NUMBER OF ITERATIONS WAS EXCEEDED.'
      ELSEIF (Cond.EQ.1) THEN
        WRITE(9,*)'CONVERGENCE HAS BEEN ACHIEVED BECAUSE '
        WRITE(9,*)'CONSECUTIVE POINTS  ARE  CLOSER  THAN ',H
      ELSEIF (Cond.EQ.2) THEN
        WRITE(9,*)'CONVERGENCE HAS BEEN ACHIEVED BECAUSE '
        WRITE(9,*)'CONSECUTIVE FUNCTION VALUES DIFFER BY ',Error
      ELSEIF (Cond.EQ.3) THEN
        WRITE(9,*)'CONVERGENCE HAS BEEN ACHIEVED BECAUSE '
        WRITE(9,*)'CONSECUTIVE POINTS  ARE  CLOSER  THAN ',H
        WRITE(9,*)'CONSECUTIVE FUNCTION VALUES DIFFER BY ',Error
      ELSEIF (Cond.EQ.4) THEN
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL BECAUSE DIVISION BY ZERO WAS
     +ENCOUNTERED.'
        IF (Error.LT.Epsilon/100.0) THEN
          WRITE(9,*)'HOWEVER, CONSECUTIVE POINTS ARE CLOSE.'
        ENDIF
      ELSEIF (Cond.EQ.5) THEN
        WRITE(9,*)'CONVERGENCE IS DOUBTFUL, H IS TOO LARGE, H = ',H
        WRITE(9,*)'IT IS POSSIBLE THAT THERE IS NO LOCAL MINIMUM.'
      ENDIF
      RETURN
      END
